from typing import Optional, Union

from pydantic import BaseModel, ConfigDict, Field

from checkout.entities.status import Status


class RedirectResponse(BaseModel):
    request_id: Union[str, int] = Field(..., alias="requestId", description="Unique transaction code for this request")
    process_url: str = Field(
        ..., alias="processUrl", description="URL to consume when the gateway requires redirection"
    )
    status: Optional[Status] = Field(..., description="Status of the transaction")
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    def is_successful(self) -> bool:
        """
        Check if the status is successful (OK).
        """
        return self.status.is_successful() if self.status else False

    def to_dict(self) -> dict:
        """
        Convert the RedirectResponse object to a dictionary.
        """
        return self.model_dump(by_alias=True, exclude_none=True)



import unittest

from checkout.entities.amount import Amount
from checkout.entities.discount import Discount
from checkout.entities.item import Item
from checkout.entities.payment import Payment
from checkout.entities.payment_modifier import PaymentModifier
from checkout.entities.person import Person


class PaymentTest(unittest.TestCase):

    def test_payment_initialization(self):
        """
        Test the initialization of a Payment object with default values.
        """
        payment = Payment(reference="REF001")
        assert payment.reference == "REF001"
        assert payment.description == ""
        assert payment.amount is None
        assert payment.allowPartial is False
        assert payment.shipping is None
        assert payment.items == []
        assert payment.recurring is None
        assert payment.payment is None
        assert payment.discount is None
        assert payment.subscribe is False
        assert payment.agreement is None
        assert payment.agreementType == ""
        assert payment.modifiers == []

    def test_payment_initialization_with_values(self):
        """
        Test the initialization of a Payment object with provided values.
        """
        amount = Amount(currency="USD", total=100.0)
        discount = Discount(code="DISC10", type="PERCENTAGE", amount=10.0, base=100.0)
        shipping = Person(document="123456789", name="John", surname="Doe")
        items = [Item(sku="ITEM001", name="Test Item", qty="1", price="50.0")]
        modifiers = [PaymentModifier(type="FEDERAL_GOVERNMENT", code="MOD001")]

        payment = Payment(
            reference="REF001",
            description="Test Payment",
            amount=amount,
            allowPartial=True,
            shipping=shipping,
            items=items,
            discount=discount,
            subscribe=True,
            agreement=123,
            agreementType="TYPE001",
            modifiers=modifiers,
        )

        assert payment.reference == "REF001"
        assert payment.description == "Test Payment"
        assert payment.amount == amount
        assert payment.allowPartial is True
        assert payment.shipping == shipping
        assert payment.items == items
        assert payment.discount == discount
        assert payment.subscribe is True
        assert payment.agreement == 123
        assert payment.agreementType == "TYPE001"
        assert payment.modifiers == modifiers

    def test_set_items(self):
        """
        Test setting items in the Payment object.
        """
        payment = Payment(reference="REF001")
        items = [{"sku": "ITEM001", "name": "Test Item", "qty": "1", "price": "50.0"}]
        payment.set_items(items)

        assert len(payment.items) == 1
        assert payment.items[0].sku == "ITEM001"

    def test_items_to_array(self):
        """
        Test converting items to an array of dictionaries.
        """
        payment = Payment(reference="REF001")
        items = [{"sku": "ITEM001", "name": "Test Item", "qty": "1", "price": "50.0"}]
        payment.set_items(items)

        items_array = payment.items_to_array()
        assert len(items_array) == 1
        assert items_array[0]["sku"] == "ITEM001"

    def test_set_modifiers(self):
        """
        Test setting modifiers in the Payment object.
        """
        payment = Payment(reference="REF001")
        modifiers = [{"type": "CUSTOM_TYPE", "code": "MOD123"}]
        payment.set_modifiers(modifiers)

        assert len(payment.modifiers) == 1
        assert payment.modifiers[0].type == "CUSTOM_TYPE"

    def test_modifiers_to_array(self):
        """
        Test converting modifiers to an array of dictionaries.
        """
        payment = Payment(reference="REF001")
        modifiers = [{"type": "CUSTOM_TYPE", "code": "MOD123"}]
        payment.set_modifiers(modifiers)

        modifiers_array = payment.modifiers_to_array()
        assert len(modifiers_array) == 1
        assert modifiers_array[0]["type"] == "CUSTOM_TYPE"

    def test_to_dict(self):
        """
        Test converting the Payment object to a dictionary.
        """
        amount = Amount(currency="USD", total=100.0)
        discount = Discount(code="DISC10", type="PERCENTAGE", amount=10.0, base=100.0)
        shipping = Person(document="123456789", name="John", surname="Doe")
        items = [Item(sku="ITEM001", name="Test Item", qty="1", price="50.0")]
        modifiers = [PaymentModifier(type="FEDERAL_GOVERNMENT", code="MOD001")]

        payment = Payment(
            reference="REF001",
            description="Test Payment",
            amount=amount,
            shipping=shipping,
            items=items,
            discount=discount,
            modifiers=modifiers,
        )

        payment_dict = payment.to_dict()
        assert payment_dict["reference"] == "REF001"
        assert payment_dict["amount"]["currency"] == "USD"
        assert payment_dict["shipping"]["name"] == "John"
        assert payment_dict["items"][0]["sku"] == "ITEM001"
        assert payment_dict["discount"]["code"] == "DISC10"
        assert payment_dict["modifiers"][0]["type"] == "FEDERAL_GOVERNMENT"



import unittest

from checkout.entities.token import Token


class TokenTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test the initialization of a Token object.
        """
        token = Token(
            token="12345",
            subtoken="67890",
            franchise="Visa",
            franchiseName="Visa International",
            issuerName="Bank of Test",
            lastDigits="1234",
            validUntil="2025-12-31",
            cvv="123",
            installments=12,
        )

        self.assertEqual(token.token, "12345")
        self.assertEqual(token.subtoken, "67890")
        self.assertEqual(token.franchise, "Visa")
        self.assertEqual(token.franchiseName, "Visa International")
        self.assertEqual(token.issuerName, "Bank of Test")
        self.assertEqual(token.lastDigits, "1234")
        self.assertEqual(token.validUntil, "2025-12-31")
        self.assertEqual(token.cvv, "123")
        self.assertEqual(token.installments, 12)

    def test_expiration_valid_date(self):
        """
        Test the expiration method with a valid date.
        """
        token = Token(validUntil="2025-12-31")
        self.assertEqual(token.expiration(), "12/25")

    def test_expiration_invalid_date(self):
        """
        Test the expiration method with an invalid date.
        """
        token = Token(validUntil="invalid-date")
        self.assertEqual(token.expiration(), "Invalid date")

    def test_expiration_empty_date(self):
        """
        Test the expiration method with an empty date.
        """
        token = Token(validUntil="")
        self.assertEqual(token.expiration(), "Invalid date")

    def test_to_dict(self):
        """
        Test the to_dict method.
        """
        token = Token(
            token="12345",
            subtoken="67890",
            franchise="Visa",
            franchiseName="Visa International",
            issuerName="Bank of Test",
            lastDigits="1234",
            validUntil="2025-12-31",
            cvv="123",
            installments=12,
        )

        expected_dict = {
            "token": "12345",
            "subtoken": "67890",
            "franchise": "Visa",
            "franchiseName": "Visa International",
            "issuerName": "Bank of Test",
            "lastDigits": "1234",
            "validUntil": "2025-12-31",
            "cvv": "123",
            "installments": 12,
        }

        self.assertEqual(token.to_dict(), expected_dict)



from typing import List, Optional, Union

from pydantic import BaseModel, Field

from checkout.entities.amount import Amount
from checkout.entities.discount import Discount
from checkout.entities.instrument import Instrument
from checkout.entities.item import Item
from checkout.entities.payment_modifier import PaymentModifier
from checkout.entities.person import Person
from checkout.entities.recurring import Recurring
from checkout.mixins.fields_mixin import FieldsMixin


class Payment(BaseModel, FieldsMixin):
    reference: str = Field(..., description="Payment reference")
    description: str = Field(default="", description="Description of the payment")
    amount: Optional[Amount] = Field(default=None, description="Amount information")
    allowPartial: bool = Field(default=False, description="Allow partial payments")
    shipping: Optional[Person] = Field(default=None, description="Shipping details")
    items: List[Item] = Field(default_factory=list, description="List of items")
    recurring: Optional[Recurring] = Field(default=None, description="Recurring payment details")
    payment: Optional[Instrument] = Field(default=None, description="Instrument payment details")
    discount: Optional[Discount] = Field(default=None, description="Discount information")
    subscribe: bool = Field(default=False, description="Subscribe flag")
    agreement: Optional[int] = Field(default=None, description="Agreement ID")
    agreementType: str = Field(default="", description="Type of agreement")
    modifiers: List[PaymentModifier] = Field(default_factory=list, description="List of payment modifiers")

    def set_items(self, items: Union[List[dict], List[Item]]) -> None:
        """
        Set the items for the payment.
        """
        self.items = [Item(**item) if isinstance(item, dict) else item for item in items]

    def items_to_array(self) -> List[dict]:
        """
        Convert the items list to an array of dictionaries.
        """
        return [item.model_dump() for item in self.items]

    def set_modifiers(self, modifiers: Union[List[dict], List[PaymentModifier]]) -> None:
        """
        Set the payment modifiers.
        """
        self.modifiers = [PaymentModifier(**mod) if isinstance(mod, dict) else mod for mod in modifiers]

    def modifiers_to_array(self) -> List[dict]:
        """
        Convert the modifiers list to an array of dictionaries.
        """
        return [modifier.model_dump() for modifier in self.modifiers]

    def to_dict(self) -> dict:
        """
        Convert the Payment object to a dictionary, including nested objects.
        """
        return {
            "reference": self.reference,
            "description": self.description,
            "amount": self.amount.model_dump() if self.amount else None,
            "allowPartial": self.allowPartial,
            "shipping": self.shipping.model_dump() if self.shipping else None,
            "items": self.items_to_array(),
            "recurring": self.recurring.model_dump() if self.recurring else None,
            "discount": self.discount.model_dump() if self.discount else None,
            "subscribe": self.subscribe,
            "agreement": self.agreement,
            "agreementType": self.agreementType,
            "modifiers": self.modifiers_to_array(),
            "fields": self.fields_to_array(),
        }



import unittest

from checkout.entities.name_value_pair import NameValuePair
from checkout.entities.subscription import Subscription
from checkout.enums.display_on_enum import DisplayOnEnum


class SubscriptionTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test the initialization of a Subscription object.
        """
        subscription = Subscription(
            reference="SUB123",
            description="Test subscription",
            customFields=[
                NameValuePair(keyword="field1", value="value1"),
                NameValuePair(keyword="field2", value="value2"),
            ],
        )

        self.assertEqual(subscription.reference, "SUB123")
        self.assertEqual(subscription.description, "Test subscription")
        self.assertEqual(len(subscription.customFields), 2)
        self.assertEqual(subscription.customFields[0].keyword, "field1")
        self.assertEqual(subscription.customFields[0].value, "value1")

    def test_to_dict(self):
        """
        Test the to_dict method.
        """
        subscription = Subscription(
            reference="SUB123",
            description="Test subscription",
            customFields=[
                NameValuePair(keyword="field1", value="value1"),
                NameValuePair(keyword="field2", value="value2"),
            ],
        )

        expected_dict = {
            "reference": "SUB123",
            "description": "Test subscription",
            "customFields": [
                {"keyword": "field1", "value": "value1", "displayOn": DisplayOnEnum.NONE},
                {"keyword": "field2", "value": "value2", "displayOn": DisplayOnEnum.NONE},
            ],
            "fields": [],
        }

        self.assertEqual(subscription.to_dict(), expected_dict)

    def test_empty_subscription(self):
        """
        Test the initialization of an empty Subscription object.
        """
        subscription = Subscription()
        self.assertEqual(subscription.reference, "")
        self.assertEqual(subscription.description, "")
        self.assertEqual(subscription.customFields, [])
        self.assertEqual(subscription.fields_to_array(), [])

    def test_fields_to_array(self):
        """
        Test the fields_to_array method from the FieldsMixin.
        """
        subscription = Subscription()
        subscription.add_field({"keyword": "key1", "value": "value1"})
        subscription.add_field({"keyword": "key2", "value": "value2"})

        expected_fields = [
            {"keyword": "key1", "value": "value1", "displayOn": DisplayOnEnum.NONE},
            {"keyword": "key2", "value": "value2", "displayOn": DisplayOnEnum.NONE},
        ]

        self.assertEqual(subscription.fields_to_array(), expected_fields)



import unittest

from checkout.entities.amount_base import AmountBase
from checkout.entities.amount_conversion import AmountConversion


class AmountConversionTest(unittest.TestCase):

    def test_amoun_conversion_initialization(self):
        to_amount = AmountBase(currency="COP", total=4000)
        from_amount = AmountBase(currency="USD", total=11)

        amount_conversion = AmountConversion(
            fromAmount=from_amount,
            toAmount=to_amount,
            factor=0.01,
        )

        assert amount_conversion.fromAmount == from_amount
        assert amount_conversion.toAmount == to_amount
        assert amount_conversion.factor == 0.01

    def test_amount_conversion_set_amount_base(self):
        amount_conversion = AmountConversion()
        amount_conversion.set_amount_base({"currency": "COP", "total": 1000})

        assert amount_conversion.toAmount.currency == "COP"
        assert amount_conversion.toAmount.total == 1000.0
        assert amount_conversion.fromAmount.currency == "COP"
        assert amount_conversion.fromAmount.total == 1000.0
        assert amount_conversion.factor == 1

    def test_amount_conversion_to_dict(self):

        amount_conversion = AmountConversion(
            fromAmount=AmountBase(currency="COP", total=4000),
            toAmount=AmountBase(currency="USD", total=11),
            factor=0.01,
        )

        assert amount_conversion.to_dict() == {
            "fromAmount": {"currency": "COP", "total": 4000.0},
            "toAmount": {"currency": "USD", "total": 11.0},
            "factor": 0.01,
        }



from pydantic import BaseModel, Field


class Address(BaseModel):
    street: str = Field(default="", description="Street address")
    city: str = Field(default="", description="City")
    state: str = Field(default="", description="State or province")
    postalCode: str = Field(default="", description="Postal code")
    country: str = Field(default="", description="Country")
    phone: str = Field(default="", description="Phone number associated with the address")

    def to_dict(self) -> dict:
        """
        Convert the address object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



import unittest

from checkout.entities.recurring import Recurring


class RecurringTest(unittest.TestCase):

    def test_recurring_initialization_with_defaults(self):
        """
        Test initialization with default values.
        """
        recurring = Recurring(
            periodicity="M",
            interval=1,
            nextPayment="2024-01-01",
        )

        assert recurring.periodicity == "M"
        assert recurring.interval == 1
        assert recurring.nextPayment == "2024-01-01"
        assert recurring.maxPeriods is None
        assert recurring.dueDate == ""
        assert recurring.notificationUrl == ""

    def test_recurring_initialization_with_values(self):
        """
        Test initialization with all values provided.
        """
        recurring = Recurring(
            periodicity="M",
            interval=1,
            nextPayment="2024-01-01",
            maxPeriods=12,
            dueDate="2024-12-31",
            notificationUrl="https://example.com/notify",
        )

        assert recurring.periodicity == "M"
        assert recurring.interval == 1
        assert recurring.nextPayment == "2024-01-01"
        assert recurring.maxPeriods == 12
        assert recurring.dueDate == "2024-12-31"
        assert recurring.notificationUrl == "https://example.com/notify"

    def test_recurring_to_dict(self):
        """
        Test the `to_dict` method.
        """
        recurring = Recurring(
            periodicity="M",
            interval=1,
            nextPayment="2024-01-01",
            maxPeriods=12,
            dueDate="2024-12-31",
            notificationUrl="https://example.com/notify",
        )

        recurring_dict = recurring.to_dict()

        expected_dict = {
            "periodicity": "M",
            "interval": 1,
            "nextPayment": "2024-01-01",
            "maxPeriods": 12,
            "dueDate": "2024-12-31",
            "notificationUrl": "https://example.com/notify",
        }

        assert recurring_dict == expected_dict

    def test_recurring_to_dict_exclude_defaults(self):
        """
        Test the `to_dict` method when optional fields are not set.
        """
        recurring = Recurring(
            periodicity="D",
            interval=7,
            nextPayment="2024-01-01",
        )

        recurring_dict = recurring.to_dict()

        expected_dict = {
            "periodicity": "D",
            "interval": 7,
            "nextPayment": "2024-01-01",
            "maxPeriods": None,
            "dueDate": "",
            "notificationUrl": "",
        }

        assert recurring_dict == expected_dict



from typing import Optional

from pydantic import BaseModel, Field

from checkout.entities.account import Account
from checkout.entities.credit import Credit
from checkout.entities.token import Token


class Instrument(BaseModel):
    bank: Optional[Account] = Field(default=None, description="Associated bank account")
    token: Optional[Token] = Field(default=None, description="Associated token")
    credit: Optional[Credit] = Field(default=None, description="Associated credit information")
    pin: str = Field(default="", description="PIN for the instrument")
    password: str = Field(default="", description="Password for the instrument")

    def to_dict(self) -> dict:
        """
        Convert the Instrument object to a dictionary.
        """
        return {
            "bank": self.bank.to_dict() if self.bank else None,
            "token": self.token.to_dict() if self.token else None,
            "credit": self.credit.to_dict() if self.credit else None,
            "pin": self.pin,
            "password": self.password,
        }



from typing import Optional

from pydantic import BaseModel, Field


class Discount(BaseModel):
    code: str = Field(..., description="The discount code")
    type: str = Field(..., description="The type of discount")
    amount: float = Field(..., description="The amount of the discount")
    base: float = Field(..., description="The base amount for the discount calculation")
    percent: Optional[float] = Field(default=None, description="The percentage of the discount, if applicable")

    def to_dict(self) -> dict:
        """
        Convert the Discount object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



import base64
import hashlib
import unittest
from unittest import mock

import pytest

from checkout.clients.authentication import Authentication
from checkout.exceptions.checkout_exception import CheckoutException


class AuthenticationTest(unittest.TestCase):

    @mock.patch("checkout.clients.authentication.random.getrandbits")
    @mock.patch("checkout.clients.authentication.datetime")
    def test_authentication_initialization(self, mock_datetime, mock_getrandbits):
        mock_now = mock.Mock()
        mock_now.isoformat.return_value = "2024-10-22T04:39:18.810868+00:00"
        mock_datetime.datetime.now.return_value = mock_now
        mock_datetime.timezone.utc = mock.Mock()
        mock_getrandbits.return_value = 927342197

        config = {
            "login": "test_login",
            "tranKey": "test_tran_key",
        }

        auth = Authentication(config)

        raw_nonce = mock_getrandbits.return_value.to_bytes(16, byteorder="big")
        expected_nonce = base64.b64encode(raw_nonce).decode("utf-8")
        expected_seed = "2024-10-22T04:39:18.810868+00:00"
        digest_input = raw_nonce + expected_seed.encode("utf-8") + config["tranKey"].encode("utf-8")
        expected_tran_key = base64.b64encode(hashlib.sha256(digest_input).digest()).decode("utf-8")

        expected_dict = {
            "login": config["login"],
            "tranKey": expected_tran_key,
            "nonce": expected_nonce,
            "seed": expected_seed,
        }

        assert auth.to_dict() == expected_dict

    def test_fails_not_login_and_tran_key_provider(self):
        with pytest.raises(CheckoutException) as exc_info:
            Authentication({})

        assert str(exc_info.value) == "No login or tranKey provided for authentication"



from typing import Optional

from pydantic import BaseModel, Field


class Recurring(BaseModel):
    periodicity: str = Field(
        ...,
        description="Frequency of the transaction (Y = annual, M = monthly, D = daily)",
    )
    interval: int = Field(..., description="Interval between payments")
    nextPayment: str = Field(default="", description="Next payment date")
    maxPeriods: Optional[int] = Field(
        default=None,
        description="Maximum times the recurrence will happen, -1 if unlimited",
    )
    dueDate: str = Field(default="", description="Due date for the recurring charge")
    notificationUrl: str = Field(default="", description="URL for sending notifications")

    def to_dict(self) -> dict:
        """
        Convert the Recurring object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



from typing import Optional

from pydantic import BaseModel, Field


class TaxDetail(BaseModel):
    kind: str = Field(..., description="The type of tax")
    amount: float = Field(..., description="The tax amount")
    base: Optional[float] = Field(default=None, description="The base amount for the tax calculation")

    def to_dict(self) -> dict:
        """
        Convert the TaxDetail object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



import unittest

from checkout.mixins.logger import Logger


class LoggerTest(unittest.TestCase):

    def setUp(self):
        """
        Create a new Logger instance for testing.
        """
        self.logger = Logger()

    def test_log_debug(self):
        """
        Test logging a debug-level message.
        """
        with self.assertLogs("P2PLogger", level="DEBUG") as log:
            self.logger.debug("Debug message", {"key": "value"})
        self.assertIn("(P2P Checkout) Debug message - Context: {'key': 'value'}", log.output[0])

    def test_log_info(self):
        """
        Test logging an info-level message.
        """
        with self.assertLogs("P2PLogger", level="INFO") as log:
            self.logger.info("Info message", {"key": "value"})
        self.assertIn("(P2P Checkout) Info message - Context: {'key': 'value'}", log.output[0])

    def test_log_warning(self):
        """
        Test logging a warning-level message.
        """
        with self.assertLogs("P2PLogger", level="WARNING") as log:
            self.logger.warning("Warning message", {"key": "value"})
        self.assertIn("(P2P Checkout) Warning message - Context: {'key': 'value'}", log.output[0])

    def test_log_error(self):
        """
        Test logging an error-level message.
        """
        with self.assertLogs("P2PLogger", level="ERROR") as log:
            self.logger.error("Error message", {"key": "value"})
        self.assertIn("(P2P Checkout) Error message - Context: {'key': 'value'}", log.output[0])

    def test_log_critical(self):
        """
        Test logging a critical-level message.
        """
        with self.assertLogs("P2PLogger", level="CRITICAL") as log:
            self.logger.critical("Critical message", {"key": "value"})
        self.assertIn("(P2P Checkout) Critical message - Context: {'key': 'value'}", log.output[0])

    def test_log_invalid_level(self):
        """
        Test logging with an invalid level (defaults to `info`).
        """
        with self.assertLogs("P2PLogger", level="INFO") as log:
            self.logger.log("invalid", "Invalid level message", {"key": "value"})
        self.assertIn("(P2P Checkout) Invalid level message - Context: {'key': 'value'}", log.output[0])

    def test_clean_context_with_valid_dict(self):
        """
        Test clean_up method with a valid dictionary.
        """
        context = {"key": "value"}
        cleaned_context = self.logger._clean_context(context)
        self.assertEqual(cleaned_context, context)

    def test_clean_context_with_none(self):
        """
        Test clean_up method with None as input.
        """
        cleaned_context = self.logger._clean_context(None)
        self.assertEqual(cleaned_context, {})

    def test_clean_context_with_invalid_type(self):
        """
        Test clean_up method with an invalid type (not a dict).
        """
        cleaned_context = self.logger._clean_context("invalid")
        self.assertEqual(cleaned_context, {})

    def test_dynamic_methods(self):
        """
        Test dynamically created logging methods (e.g., debug, info, etc.).
        """
        with self.assertLogs("P2PLogger", level="DEBUG") as log:
            self.logger.debug("Dynamically created debug method", {"key": "value"})
        self.assertIn("(P2P Checkout) Dynamically created debug method - Context: {'key': 'value'}", log.output[0])

    def test_invalid_dynamic_method(self):
        """
        Test accessing an invalid dynamic method raises an AttributeError.
        """
        with self.assertRaises(AttributeError):
            self.logger.invalid_method("This should fail")



from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field, field_validator

from checkout.entities.account import Account
from checkout.entities.name_value_pair import NameValuePair
from checkout.entities.status import Status
from checkout.entities.token import Token


class SubscriptionInformation(BaseModel):
    type: str = Field(default="", description="Type of subscription (e.g., token, account)")
    status: Optional[Status] = Field(default=None, description="Status information")
    instrument: List[NameValuePair] = Field(default_factory=list, description="Instrument details as name-value pairs")

    @field_validator("instrument", mode="before")
    @classmethod
    def validate_instrument(
        cls, instrument_data: Union[List[dict], List[NameValuePair], dict, None]
    ) -> List[NameValuePair]:
        """
        Ensure the instrument field is always a list of NameValuePair objects.
        """
        if not instrument_data:
            return []

        if isinstance(instrument_data, dict):
            if "item" in instrument_data:
                instrument_data = instrument_data["item"]
            else:
                instrument_data = [instrument_data]

        if not isinstance(instrument_data, list):
            raise ValueError("Instrument data must be a list of dictionaries or NameValuePair objects.")

        return [nvp if isinstance(nvp, NameValuePair) else NameValuePair(**nvp) for nvp in instrument_data]

    def set_instrument(self, instrument_data: Union[dict, List[dict]]) -> None:
        """
        Set the instrument data as a list of NameValuePair objects.
        """
        self.instrument = []
        if isinstance(instrument_data, dict) and "item" in instrument_data:
            instrument_data = instrument_data["item"]

        for nvp_data in instrument_data:
            nvp_data = nvp_data if isinstance(nvp_data, NameValuePair) else NameValuePair(**nvp_data)
            self.instrument.append(nvp_data)

    def instrument_to_list(self) -> List[dict]:
        """
        Convert the instrument to a list of dictionaries.
        """
        return [nvp.to_dict() for nvp in self.instrument]

    def parse_instrument(self) -> Optional[Union[Account, Token]]:
        """
        Parse the instrument as the proper entity (Account or Token) or return None.
        """
        if not self.instrument:
            return None

        data: Dict[str, Any] = {}
        if self.status:
            data["status"] = self.status

        for nvp in self.instrument:
            data[nvp.keyword] = nvp.value

        if self.type == "token":
            return Token(
                token=data.get("token", ""),
                subtoken=data.get("subtoken", ""),
                franchise=data.get("franchise", ""),
                franchiseName=data.get("franchiseName", ""),
                issuerName=data.get("issuerName", ""),
                lastDigits=data.get("lastDigits", ""),
                validUntil=data.get("validUntil", ""),
                cvv=data.get("cvv", ""),
                installments=data.get("installments", 0),
            )

        elif self.type == "account":
            return Account(
                bankCode=data.get("bankCode", ""),
                bankName=data.get("bankName", ""),
                accountType=data.get("accountType", ""),
                accountNumber=data.get("accountNumber", ""),
                status=self.status if isinstance(self.status, Status) else None,
            )

        return None

    def to_dict(self) -> dict:
        """
        Convert the SubscriptionInformation object to a dictionary.
        """
        return {
            "type": self.type,
            "status": self.status.to_dict() if self.status else None,
            "instrument": self.instrument_to_list(),
        }



import json

from requests import Response


class HttpException(Exception):
    """Base class for HTTP-related exceptions."""

    @classmethod
    def from_response(cls, response: Response) -> "HttpException":
        """
        Create an exception from a Response object.
        """
        try:
            error_details = response.json()
        except json.JSONDecodeError:
            error_details = {"status_code": response.status_code, "message": response.text}

        return cls(
            json.dumps(
                {
                    "status_code": response.status_code,
                    "error_details": error_details,
                }
            )
        )


class ClientErrorException(HttpException):
    """Exception for 4xx HTTP errors."""


class ServerErrorException(HttpException):
    """Exception for 5xx HTTP errors."""



from enum import Enum


class StatusEnum(str, Enum):
    OK = "OK"
    FAILED = "FAILED"
    APPROVED = "APPROVED"
    APPROVED_PARTIAL = "APPROVED_PARTIAL"
    REJECTED = "REJECTED"
    PENDING = "PENDING"
    PENDING_VALIDATION = "PENDING_VALIDATION"
    REFUNDED = "REFUNDED"
    ERROR = "ERROR"
    UNKNOWN = "UNKNOWN"



import logging
from typing import Any, Callable, Dict, Optional, cast


class Logger:
    """
    A wrapper for logging with dynamic method handling for different log levels.
    """

    def __init__(self, logger: Optional[logging.Logger] = None) -> None:
        """
        Initialize the Logger instance.

        :param logger: An optional `logging.Logger` instance. If not provided, a default logger is created.
        """
        self.logger: logging.Logger = logger if logger else self._create_default_logger()

    def log(self, level: str, message: str, context: Optional[Dict[str, Any]] = None) -> None:
        """
        Log a message with the specified level.

        :param level: Log level as a string (e.g., 'debug', 'info', etc.).
        :param message: The message to log.
        :param context: Additional context to include in the log.
        """
        log_func = self._get_log_func(level)
        cleaned_context = self._clean_context(context)
        log_func(f"(P2P Checkout) {message} - Context: {cleaned_context}")

    def _get_log_func(self, level: str) -> Callable[[str], None]:
        """
        Retrieve the logging function for the specified level.

        :param level: Log level as a string.
        :return: The logging function corresponding to the level, or `info` as fallback.
        """
        return getattr(self.logger, level.lower(), self.logger.info)

    def __getattr__(self, name: str) -> Callable[[str, Optional[Dict[str, Any]]], None]:
        """
        Dynamically handle logging methods like `debug`, `info`, etc.

        :param name: The name of the method being called.
        :return: A callable function that logs with the specified level.
        """
        if name in {"debug", "info", "warning", "error", "critical"}:
            return cast(
                Callable[[str, Optional[Dict[str, Any]]], None],
                lambda message, context=None: self.log(name, message, context),
            )
        raise AttributeError(f"'Logger' object has no attribute '{name}'")

    @staticmethod
    def _clean_context(context: Optional[Dict[str, Any]]) -> Dict[str, Any]:
        """
        Clean up the context by ensuring it's a dictionary.

        :param context: The context to clean up.
        :return: A cleaned dictionary (empty if input is None or invalid).
        """
        return context if isinstance(context, dict) else {}

    @staticmethod
    def _create_default_logger() -> logging.Logger:
        """
        Create and configure a default logger.

        :return: A `logging.Logger` instance.
        """
        logger = logging.getLogger("P2PLogger")
        logger.setLevel(logging.DEBUG)
        handler = logging.StreamHandler()
        formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
        handler.setFormatter(formatter)
        logger.addHandler(handler)
        return logger



from datetime import datetime

from pydantic import BaseModel, Field


class Token(BaseModel):
    token: str = Field(default="", description="Unique token identifier")
    subtoken: str = Field(default="", description="Secondary token identifier")
    franchise: str = Field(default="", description="Franchise associated with the token")
    franchiseName: str = Field(default="", description="Name of the franchise")
    issuerName: str = Field(default="", description="Name of the issuer")
    lastDigits: str = Field(default="", description="Last digits of the card/token")
    validUntil: str = Field(default="", description="Expiration date in ISO format")
    cvv: str = Field(default="", description="CVV associated with the token")
    installments: int = Field(default=0, description="Number of installments")

    def expiration(self) -> str:
        """
        Convert valid_until to 'mm/yy' format for expiration date.
        """
        try:
            expiration_date = datetime.strptime(self.validUntil, "%Y-%m-%d")
            return expiration_date.strftime("%m/%y")
        except ValueError:
            return "Invalid date"

    def to_dict(self) -> dict:
        """
        Convert the Token object to a dictionary using the Pydantic model_dump method.
        """
        return self.model_dump()



from typing import Dict, List, Optional, Union

from checkout.entities.name_value_pair import NameValuePair


class FieldsMixin:
    fields: List[NameValuePair] = []

    def get_fields(self) -> List[NameValuePair]:
        """
        Return the list of NameValuePair objects.
        """
        return self.fields

    def set_fields(self, fields_data: Union[List[Dict], Dict]) -> None:
        """
        Set the fields based on the provided data.
        """
        if isinstance(fields_data, dict) and "item" in fields_data:
            fields_data = fields_data["item"]

        self.fields = []
        for nvp in fields_data:
            self.fields.append(nvp if isinstance(nvp, NameValuePair) else NameValuePair(**nvp))

    def fields_to_array(self) -> List[Dict]:
        """
        Convert the fields to a list of dictionaries.
        """
        return [field.to_dict() for field in self.fields if isinstance(field, NameValuePair)]

    def fields_to_key_value(self, nvps: Optional[List[NameValuePair]] = None) -> Dict[str, Union[str, list, dict]]:
        """
        Convert the fields to a key-value pair dictionary.
        """
        nvps_data = nvps if nvps is not None else self.fields

        return {field.keyword: field.value for field in nvps_data if isinstance(field, NameValuePair)}

    def add_field(self, nvp: Union[Dict, NameValuePair]) -> None:
        """
        Add a new NameValuePair to the fields.
        """
        name_value_pair = nvp if isinstance(nvp, NameValuePair) else NameValuePair(**nvp)
        self.fields.append(name_value_pair)



from typing import List, Optional

from pydantic import BaseModel, Field

from checkout.entities.name_value_pair import NameValuePair
from checkout.mixins.fields_mixin import FieldsMixin


class Subscription(BaseModel, FieldsMixin):
    reference: str = Field(default="", description="Reference for the subscription")
    description: str = Field(default="", description="Description of the subscription")
    customFields: Optional[List[NameValuePair]] = Field(
        default_factory=lambda: [], description="Additional fields for the subscription"
    )

    def to_dict(self) -> dict:
        """
        Convert the subscription object to a dictionary including fields.
        """
        data = self.model_dump()
        data["fields"] = self.fields_to_array()
        return data



import unittest

from checkout.entities.amount_base import AmountBase


class AmountBaseTest(unittest.TestCase):

    def test_amount_base_inizalitation(self):

        amount_base = AmountBase(currency="USD", total=10000)

        assert amount_base.currency == "USD"
        assert amount_base.total == 10000

    def test_amount_base_default_values(self):
        amount_base = AmountBase(total=10000)

        assert amount_base.currency == "COP"
        assert amount_base.total == 10000

    def test_amount_base_to_dict(self):
        amount_base = AmountBase(currency="COP", total=20000)

        expected = {"currency": "COP", "total": 20000.0}
        assert amount_base.to_dict() == expected



from typing import List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field

from checkout.entities.status import Status
from checkout.entities.subscription_information import SubscriptionInformation
from checkout.entities.transaction import Transaction
from checkout.messages.requests.redirect import RedirectRequest


class InformationResponse(BaseModel):

    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    request_id: Union[str, int] = Field(..., alias="requestId", description="Unique identifier for the request")
    status: Optional[Status] = Field(default=None, description="Status of the request")
    request: Optional[RedirectRequest] = Field(default=None, description="Redirect request details")
    payment: Optional[List[Transaction]] = Field(default=None, description="List of payment transactions")
    subscription: Optional[SubscriptionInformation] = Field(default=None, description="Subscription details")

    def set_payment(self, payments: List[dict]) -> None:
        """
        Set the payment transactions.
        """
        if payments:
            self.payment = []
            if isinstance(payments, dict) and "transaction" in payments:
                payments = payments["transaction"]

            for payment_data in payments:
                self.payment.append(Transaction(**payment_data))

    def last_transaction(self, approved: bool = False) -> Optional[Transaction]:
        """
        Get the last transaction made in the session.
        If approved is True, returns the last approved transaction.
        """
        if not self.payment:
            return None

        if not approved:
            return self.payment[-1] if self.payment else None

        for transaction in self.payment:
            if transaction.is_approved():
                return transaction

        return None

    def last_approved_transaction(self) -> Optional[Transaction]:
        """
        Get the last approved transaction.
        """
        return self.last_transaction(approved=True)

    def last_authorization(self) -> str:
        """
        Returns the last authorization associated with the session.
        """
        last_approved = self.last_approved_transaction()
        return last_approved.authorization if last_approved else ""

    def to_dict(self) -> dict:
        """
        Convert the RedirectInformation object to a dictionary.
        """
        return {
            "requestId": self.request_id,
            "status": self.status.to_dict() if self.status else None,
            "request": self.request.to_dict() if self.request else None,
            "payment": [transaction.to_dict() for transaction in self.payment] if self.payment else None,
            "subscription": self.subscription.to_dict() if self.subscription else None,
        }



from typing import Optional

from pydantic import BaseModel, Field

from checkout.entities.address import Address


class Person(BaseModel):
    document: str = Field(default="", description="Document number of the person")
    documentType: str = Field(default="", description="Type of document (e.g., ID, Passport)")
    name: str = Field(default="", description="First name of the person")
    surname: str = Field(default="", description="Last name of the person")
    company: str = Field(default="", description="Company name if applicable")
    email: Optional[str] = Field(default="", description="Email address of the person")
    mobile: str = Field(default="", description="Mobile number of the person")
    address: Optional[Address] = Field(default=None, description="Address information")

    def is_business(self) -> bool:
        """
        Check if the person is representing a business based on their document type.
        """
        return bool(self.documentType and self._business_document(self.documentType))

    @staticmethod
    def _business_document(document_type: str) -> bool:
        """
        Placeholder for business document validation.
        Replace this with actual logic similar to DocumentHelper::businessDocument.
        """
        business_document_types = {"TIN", "VAT", "EIN"}
        return document_type in business_document_types

    def to_dict(self) -> dict:
        """
        Convert the person object to a dictionary using the new `model_dump` method.
        """
        return self.model_dump(exclude_none=True)



from typing import Optional

from pydantic import BaseModel, ConfigDict, Field

from checkout.decorators.convert_to_boolean import convert_booleans_to_strings
from checkout.entities.dispersion_payment import DispersionPayment
from checkout.entities.person import Person
from checkout.entities.subscription import Subscription


class RedirectRequest(BaseModel):

    locale: str = Field(default="es_CO", description="Locale of the request")
    payer: Optional[Person] = Field(default=None, description="Information about the payer")
    buyer: Optional[Person] = Field(default=None, description="Information about the buyer")
    payment: Optional[DispersionPayment] = Field(default=None, description="Payment details")
    subscription: Optional[Subscription] = Field(default=None, description="Subscription details")
    return_url: str = Field(..., description="URL to return to after processing", alias="returnUrl")
    payment_method: Optional[str] = Field(default=None, description="Payment method to be used", alias="paymentMethod")
    cancel_url: Optional[str] = Field(default=None, description="URL to return to if canceled", alias="cancelUrl")
    ip_address: str = Field(..., description="IP address of the user", alias="ipAddress")
    user_agent: str = Field(..., description="User agent of the user's browser", alias="userAgent")
    expiration: Optional[str] = Field(default=None, description="Expiration date for the request")
    capture_address: bool = Field(default=False, description="Whether to capture the address", alias="captureAddress")
    skip_result: bool = Field(default=False, description="Whether to skip showing results", alias="skipResult")
    no_buyer_fill: bool = Field(
        default=False, description="Whether to avoid pre-filling buyer data", alias="noBuyerFill"
    )

    model_config = ConfigDict(populate_by_name=True, arbitrary_types_allowed=True)

    @convert_booleans_to_strings
    def to_dict(self) -> dict:
        """
        Convert the RedirectRequest object to a dictionary using Pydantic's dict method.
        """
        return self.model_dump(exclude_none=True, by_alias=True)



import unittest

from checkout.entities.amount_conversion import AmountConversion
from checkout.entities.discount import Discount
from checkout.entities.name_value_pair import NameValuePair
from checkout.entities.status import Status
from checkout.entities.transaction import Transaction
from checkout.enums.status_enum import StatusEnum


class TransactionTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test initialization of Transaction with minimal data.
        """
        transaction = Transaction(reference="test_ref")
        self.assertEqual(transaction.reference, "test_ref")
        self.assertEqual(transaction.internal_reference, "")
        self.assertEqual(transaction.payment_method, "")
        self.assertIsNone(transaction.amount)
        self.assertIsNone(transaction.status)
        self.assertFalse(transaction.refunded)
        self.assertEqual(transaction.processor_fields, [])

    def test_is_successful(self):
        """
        Test is_successful method with different status values.
        """
        transaction = Transaction(reference="test_ref", status=Status(status=StatusEnum.ERROR, reason="XX"))
        self.assertFalse(transaction.is_successful())

        transaction.status.status = StatusEnum.OK
        self.assertTrue(transaction.is_successful())

    def test_is_approved(self):
        """
        Test is_approved method with different status values.
        """
        transaction = Transaction(reference="test_ref", status=Status(status=StatusEnum.APPROVED, reason="00"))
        self.assertTrue(transaction.is_approved())

        transaction.status.status = StatusEnum.REJECTED
        self.assertFalse(transaction.is_approved())

    def test_set_processor_fields(self):
        """
        Test setting processor fields from a list of dictionaries.
        """
        data = [{"keyword": "key1", "value": "value1"}, {"keyword": "key2", "value": "value2"}]
        transaction = Transaction(reference="test_ref")
        transaction.set_processor_fields(data)

        self.assertEqual(len(transaction.processor_fields), 2)
        self.assertEqual(transaction.processor_fields[0].keyword, "key1")
        self.assertEqual(transaction.processor_fields[0].value, "value1")
        self.assertEqual(transaction.processor_fields[1].keyword, "key2")
        self.assertEqual(transaction.processor_fields[1].value, "value2")

    def test_processor_fields_to_array(self):
        """
        Test conversion of processor fields to a list of dictionaries.
        """
        data = [NameValuePair(keyword="key1", value="value1"), NameValuePair(keyword="key2", value="value2")]
        transaction = Transaction(reference="test_ref", processor_fields=data)
        result = transaction.processor_fields_to_array()

        expected = [
            {"keyword": "key1", "value": "value1", "displayOn": "none"},
            {"keyword": "key2", "value": "value2", "displayOn": "none"},
        ]
        self.assertEqual(result, expected)

    def test_set_processor_fields_with_item_key(self):
        """
        Test set_processor_fields with a dictionary containing an 'item' key.
        """

        processor_fields_data = {
            "item": [
                {"keyword": "key1", "value": "value1"},
                {"keyword": "key2", "value": "value2"},
            ]
        }

        transaction = Transaction(reference="testReference")

        transaction.set_processor_fields(processor_fields_data)

        self.assertEqual(len(transaction.processor_fields), 2)
        self.assertEqual(transaction.processor_fields[0].keyword, "key1")
        self.assertEqual(transaction.processor_fields[0].value, "value1")
        self.assertEqual(transaction.processor_fields[1].keyword, "key2")
        self.assertEqual(transaction.processor_fields[1].value, "value2")

    def test_additional_data(self):
        """
        Test parsing processor fields as a key-value dictionary.
        """
        data = [NameValuePair(keyword="key1", value="value1"), NameValuePair(keyword="key2", value="value2")]
        transaction = Transaction(reference="test_ref", processor_fields=data)
        additional_data = transaction.additional_data()

        expected = {
            "key1": "value1",
            "key2": "value2",
        }
        self.assertEqual(additional_data, expected)

    def test_to_dict(self):
        """
        Test conversion of Transaction object to a dictionary.
        """
        status = Status(status=StatusEnum.APPROVED, reason="Test reason")
        amount = AmountConversion(currency="USD", total=100.0)
        discount = Discount(code="DISCOUNT1", type="PERCENT", amount=10.0, base=100.0)
        processor_fields = [
            NameValuePair(keyword="key1", value="value1"),
            NameValuePair(keyword="key2", value="value2"),
        ]

        transaction = Transaction(
            reference="test_ref",
            internal_reference="internal_ref",
            payment_method="card",
            payment_method_name="Credit Card",
            issuer_name="Test Bank",
            amount=amount,
            authorization="12345",
            receipt="54321",
            franchise="VISA",
            refunded=True,
            discount=discount,
            processor_fields=processor_fields,
            status=status,
        )

        result = transaction.to_dict()

        expected = {
            "status": status.to_dict(),
            "internalReference": "internal_ref",
            "paymentMethod": "card",
            "paymentMethodName": "Credit Card",
            "issuerName": "Test Bank",
            "amount": amount.to_dict(),
            "authorization": "12345",
            "reference": "test_ref",
            "receipt": "54321",
            "franchise": "VISA",
            "refunded": True,
            "discount": discount.to_dict(),
            "processorFields": [
                {"keyword": "key1", "value": "value1", "displayOn": "none"},
                {"keyword": "key2", "value": "value2", "displayOn": "none"},
            ],
        }
        self.assertEqual(result, expected)



import unittest

from checkout.entities.status import Status
from checkout.entities.subscription_information import SubscriptionInformation
from checkout.entities.transaction import Transaction
from checkout.enums.status_enum import StatusEnum
from checkout.messages.requests.redirect import RedirectRequest
from checkout.messages.responses.information import InformationResponse


class InformationTest(unittest.TestCase):

    def test_initialization_with_all_fields(self):
        """
        Test initialization of Information with all fields provided.
        """
        status = Status(status=StatusEnum.OK, reason="Request successful")
        request = RedirectRequest(
            locale="en_US",
            return_url="https://example.com/return",
            ip_address="192.168.0.1",
            user_agent="TestAgent",
        )
        transaction = Transaction(
            reference="REF001",
            internal_reference="INT001",
            payment_method="CreditCard",
            payment_method_name="Visa",
            issuer_name="Bank A",
            authorization="AUTH001",
            receipt="REC001",
            franchise="VISA",
            refunded=False,
        )
        subscription = SubscriptionInformation(type="token")

        information = InformationResponse(
            request_id="REQ123",
            status=status,
            request=request,
            payment=[transaction],
            subscription=subscription,
        )

        self.assertEqual(information.request_id, "REQ123")
        self.assertEqual(information.status.reason, "Request successful")
        self.assertEqual(information.request.return_url, "https://example.com/return")
        self.assertEqual(len(information.payment), 1)
        self.assertEqual(information.subscription.type, "token")

    def test_initialization_with_defaults(self):
        """
        Test initialization of Information with default values.
        """
        information = InformationResponse(request_id="REQ123")

        self.assertEqual(information.request_id, "REQ123")
        self.assertIsNone(information.status)
        self.assertIsNone(information.request)
        self.assertIsNone(information.payment)
        self.assertIsNone(information.subscription)

    def test_set_payment(self):
        """
        Test setting payment transactions.
        """
        payments_data = [
            {"reference": "REF001", "internalReference": "INT001", "authorization": "AUTH001"},
            {"reference": "REF002", "internalReference": "INT002", "authorization": "AUTH002"},
        ]

        information = InformationResponse(request_id="REQ123")
        information.set_payment(payments_data)

        self.assertEqual(len(information.payment), 2)
        self.assertEqual(information.payment[0].reference, "REF001")
        self.assertEqual(information.payment[1].reference, "REF002")

    def test_last_transaction(self):
        """
        Test retrieving the last transaction.
        """
        transaction1 = Transaction(reference="REF001", authorization="AUTH001", refunded=False)
        transaction2 = Transaction(reference="REF002", authorization="AUTH002", refunded=False)

        information = InformationResponse(request_id="REQ123", payment=[transaction1, transaction2])

        last_transaction = information.last_transaction()
        self.assertEqual(last_transaction.reference, "REF002")

    def test_last_approved_transaction(self):
        """
        Test retrieving the last approved transaction.
        """
        status_approved = Status(status=StatusEnum.APPROVED, reason="Approved")
        transaction1 = Transaction(reference="REF001", authorization="AUTH001", refunded=False)
        transaction2 = Transaction(reference="REF002", authorization="AUTH002", refunded=False, status=status_approved)

        information = InformationResponse(request_id="REQ123", payment=[transaction1, transaction2])

        last_approved_transaction = information.last_approved_transaction()
        self.assertEqual(last_approved_transaction.reference, "REF002")

    def test_last_authorization(self):
        """
        Test retrieving the last authorization.
        """
        status_approved = Status(status=StatusEnum.APPROVED, reason="Approved")
        transaction = Transaction(reference="REF001", authorization="AUTH001", refunded=False, status=status_approved)

        information = InformationResponse(request_id="REQ123", payment=[transaction])

        last_authorization = information.last_authorization()
        self.assertEqual(last_authorization, "AUTH001")

    def test_to_dict(self):
        """
        Test converting the Information object to a dictionary.
        """
        status = Status(status=StatusEnum.OK, reason="Request successful")
        transaction = Transaction(reference="REF001", authorization="AUTH001")
        subscription = SubscriptionInformation(type="token")

        information = InformationResponse(
            request_id="REQ123",
            status=status,
            payment=[transaction],
            subscription=subscription,
        )

        expected_dict = {
            "requestId": "REQ123",
            "status": status.to_dict(),
            "request": None,
            "payment": [transaction.to_dict()],
            "subscription": subscription.to_dict(),
        }

        self.assertEqual(information.to_dict(), expected_dict)

    def test_set_payment_with_nested_transactions(self):
        """
        Test `set_payment` when the input contains nested transactions in a dictionary.
        """
        information = InformationResponse(
            request_id="12345",
            status=Status(status="OK", reason="Success"),
        )
        nested_payments = {
            "transaction": [
                {"reference": "TX001", "amount": {"currency": "USD", "total": 100}},
                {"reference": "TX002", "amount": {"currency": "USD", "total": 200}},
            ]
        }
        information.set_payment(nested_payments)
        self.assertEqual(len(information.payment), 2)
        self.assertEqual(information.payment[0].reference, "TX001")
        self.assertEqual(information.payment[1].reference, "TX002")

    def test_last_transaction_no_payments(self):
        """
        Test `last_transaction` when there are no payments.
        """
        information = InformationResponse(
            request_id="12345",
            status=Status(status="OK", reason="Success"),
        )
        self.assertIsNone(information.last_transaction())

    def test_set_payment_with_empty_list(self):
        """
        Test `set_payment` when the input is an empty list.
        """
        information = InformationResponse(
            request_id="12345",
            status=Status(status="OK", reason="Success"),
        )
        information.set_payment([])
        self.assertIsNone(information.payment)

    def test_last_transaction_no_approved(self):
        """
        Test `last_transaction` when `approved=True` and no transactions are approved.
        """
        information = InformationResponse(
            request_id="12345",
            status=Status(status=StatusEnum.OK, reason="Success"),
            payment=[
                Transaction(reference="TX001", status=Status(status=StatusEnum.FAILED, reason="Failed")),
                Transaction(reference="TX002", status=Status(status=StatusEnum.FAILED, reason="Failed")),
            ],
        )
        print(information.last_transaction(approved=True))
        self.assertIsNone(information.last_transaction(approved=True))



import unittest

from checkout.entities.account import Account
from checkout.entities.name_value_pair import NameValuePair
from checkout.entities.status import Status
from checkout.entities.subscription_information import SubscriptionInformation
from checkout.entities.token import Token
from checkout.enums.status_enum import StatusEnum


class SubsCriptionInformationTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test initialization with default values.
        """
        subscription = SubscriptionInformation()
        self.assertEqual(subscription.type, "")
        self.assertIsNone(subscription.status)
        self.assertEqual(subscription.instrument, [])

    def test_set_instrument_with_dict(self):
        """
        Test setting instrument with dictionary data.
        """
        instrument_data = {"item": [{"keyword": "key1", "value": "value1"}]}
        subscription = SubscriptionInformation()
        subscription.set_instrument(instrument_data)

        self.assertEqual(len(subscription.instrument), 1)
        self.assertEqual(subscription.instrument[0].keyword, "key1")
        self.assertEqual(subscription.instrument[0].value, "value1")

    def test_set_instrument_with_list(self):
        """
        Test setting instrument with a list of dictionaries.
        """
        instrument_data = [{"keyword": "key1", "value": "value1"}]
        subscription = SubscriptionInformation()
        subscription.set_instrument(instrument_data)

        self.assertEqual(len(subscription.instrument), 1)
        self.assertEqual(subscription.instrument[0].keyword, "key1")
        self.assertEqual(subscription.instrument[0].value, "value1")

    def test_instrument_to_list(self):
        """
        Test converting instrument to a list of dictionaries.
        """
        instrument_data = [
            NameValuePair(keyword="key1", value="value1"),
            NameValuePair(keyword="key2", value="value2"),
        ]
        subscription = SubscriptionInformation(instrument=instrument_data)
        instrument_list = subscription.instrument_to_list()

        self.assertEqual(len(instrument_list), 2)
        self.assertEqual(instrument_list[0]["keyword"], "key1")
        self.assertEqual(instrument_list[1]["value"], "value2")

    def test_parse_instrument_as_token(self):
        """
        Test parsing instrument as a Token.
        """

        instrument_data = [
            NameValuePair(keyword="token", value="12345"),
            NameValuePair(keyword="subtoken", value="54321"),
            NameValuePair(keyword="franchise", value="visa"),
            NameValuePair(keyword="franchiseName", value="Visa"),
            NameValuePair(keyword="validUntil", value="2025-12-31"),
        ]
        subscription = SubscriptionInformation(
            type="token", status=Status.quick(status=StatusEnum.OK, reason="00"), instrument=instrument_data
        )
        result = subscription.parse_instrument()

        self.assertIsInstance(result, Token)
        self.assertEqual(result.token, "12345")
        self.assertEqual(result.subtoken, "54321")
        self.assertEqual(result.franchise, "visa")
        self.assertEqual(result.validUntil, "2025-12-31")

    def test_parse_instrument_as_account(self):
        """
        Test parsing instrument as an Account.
        """

        instrument_data = [
            NameValuePair(keyword="bankCode", value="001"),
            NameValuePair(keyword="bankName", value="Bank A"),
            NameValuePair(keyword="accountType", value="savings"),
            NameValuePair(keyword="accountNumber", value="123456789"),
        ]
        subscription = SubscriptionInformation(
            type="account", status=Status.quick(status=StatusEnum.OK, reason="00"), instrument=instrument_data
        )
        result = subscription.parse_instrument()

        self.assertIsInstance(result, Account)
        self.assertEqual(result.bankCode, "001")
        self.assertEqual(result.bankName, "Bank A")
        self.assertEqual(result.accountType, "savings")
        self.assertEqual(result.accountNumber, "123456789")

    def test_set_instrument_list_of_dicts(self):
        """
        Test setting instrument when input is a list of dictionaries.
        """
        instrument_data = [{"keyword": "key1", "value": "value1"}]
        subscription = SubscriptionInformation()
        subscription.set_instrument(instrument_data)

        self.assertEqual(len(subscription.instrument), 1)
        self.assertIsInstance(subscription.instrument[0], NameValuePair)
        self.assertEqual(subscription.instrument[0].keyword, "key1")
        self.assertEqual(subscription.instrument[0].value, "value1")

    def test_set_instrument_list_of_name_value_pairs(self):
        """
        Test setting instrument when input is a list of NameValuePair objects.
        """
        nvp = NameValuePair(keyword="key1", value="value1")
        subscription = SubscriptionInformation()
        subscription.set_instrument([nvp])

        self.assertEqual(len(subscription.instrument), 1)
        self.assertIsInstance(subscription.instrument[0], NameValuePair)
        self.assertEqual(subscription.instrument[0].keyword, "key1")
        self.assertEqual(subscription.instrument[0].value, "value1")

    def test_parse_instrument_no_type(self):
        """
        Test parsing instrument with no type set.
        """
        instrument_data = [
            NameValuePair(keyword="key1", value="value1"),
        ]
        subscription = SubscriptionInformation(instrument=instrument_data)
        result = subscription.parse_instrument()

        self.assertIsNone(result)

    def test_to_dict(self):
        """
        Test conversion of SubscriptionInformation to dictionary.
        """

        instrument_data = [
            NameValuePair(keyword="key1", value="value1"),
        ]
        subscription = SubscriptionInformation(
            type="token", status=Status.quick(status=StatusEnum.OK, reason="00"), instrument=instrument_data
        )
        result_dict = subscription.to_dict()

        expected_dict = {
            "type": "token",
            "status": {"status": "OK", "reason": "00", "message": "", "date": None},
            "instrument": [{"keyword": "key1", "value": "value1", "displayOn": "none"}],
        }
        self.assertEqual(result_dict, expected_dict)

    def test_parse_instrument_empty_instrument(self):
        """
        Test parsing instrument when instrument list is empty.
        """
        subscription = SubscriptionInformation(type="account", instrument=[])
        result = subscription.parse_instrument()
        self.assertIsNone(result, "Expected None when instrument list is empty.")

    def test_empty_instrument_to_dict(self):
        """
        Test conversion to dictionary with an empty instrument list.
        """
        subscription = SubscriptionInformation(type="account")
        result_dict = subscription.to_dict()

        expected_dict = {
            "type": "account",
            "status": None,
            "instrument": [],
        }
        self.assertEqual(result_dict, expected_dict)

    def test_validate_instrument_none(self):
        """
        Test when instrument_data is None or empty.
        """
        result = SubscriptionInformation.validate_instrument(None)
        self.assertEqual(result, [])

    def test_validate_instrument_single_dict(self):
        """
        Test when instrument_data is a single dictionary.
        """
        data = {"keyword": "key1", "value": "value1"}
        result = SubscriptionInformation.validate_instrument(data)
        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], NameValuePair)
        self.assertEqual(result[0].keyword, "key1")
        self.assertEqual(result[0].value, "value1")

    def test_validate_instrument_list_of_dicts(self):
        """
        Test when instrument_data is a list of dictionaries.
        """
        data = [{"keyword": "key1", "value": "value1"}, {"keyword": "key2", "value": "value2"}]
        result = SubscriptionInformation.validate_instrument(data)
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], NameValuePair)
        self.assertEqual(result[0].keyword, "key1")
        self.assertEqual(result[1].keyword, "key2")

    def test_validate_instrument_with_item_key(self):
        """
        Test when instrument_data contains an 'item' key with a list.
        """
        data = {"item": [{"keyword": "key1", "value": "value1"}, {"keyword": "key2", "value": "value2"}]}
        result = SubscriptionInformation.validate_instrument(data)
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], NameValuePair)
        self.assertEqual(result[0].keyword, "key1")
        self.assertEqual(result[1].keyword, "key2")

    def test_validate_instrument_invalid_data(self):
        """
        Test when instrument_data is not a list or dict.
        """
        data = "invalid_data"
        with self.assertRaises(ValueError):
            SubscriptionInformation.validate_instrument(data)



from typing import Any, Optional

from pydantic import BaseModel, Field

from checkout.enums.display_on_enum import DisplayOnEnum


class NameValuePair(BaseModel):
    keyword: str = Field(..., description="The keyword associated with the value")
    value: Any = Field(default=None, description="The value, which can be a string, list, or dict")
    displayOn: Optional[DisplayOnEnum] = Field(
        default=DisplayOnEnum.NONE, description="Display setting for the keyword"
    )

    def to_dict(self) -> dict:
        """
        Convert the NameValuePair object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



import logging
from typing import Any, Dict, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, Field, model_validator

from checkout.clients.authentication import Authentication
from checkout.clients.http_client import HttpClient
from checkout.contracts.carrier import Carrier


class Settings(BaseModel):
    """
    Configuration class for PlaceToPay integration using Pydantic.
    """

    model_config = {"arbitrary_types_allowed": True}

    base_url: str = Field(..., description="Base URL for the API")
    timeout: int = Field(default=15, description="Request timeout in seconds")
    login: str = Field(..., description="API login key")
    tranKey: str = Field(..., description="API transaction key")
    headers: Dict[str, str] = Field(default_factory=dict, description="Additional HTTP headers")
    auth_additional: Dict[str, Any] = Field(default_factory=dict, description="Additional authentication data")
    loggerConfig: Optional[Dict[str, Any]] = Field(default=None, description="Logger configuration")
    p2p_client: Optional[HttpClient] = Field(default=None, description="Optional pre-configured HttpClient")
    p2p_logger: Optional[logging.Logger] = Field(default=None)
    _carrier_instance: Optional[Carrier] = None

    @model_validator(mode="before")
    @classmethod
    def validate_base_url(cls, values: Dict[str, Any]) -> Dict[str, Any]:
        """
        Ensure the base_url ends with a slash and is valid.
        """
        base_url = values.get("base_url")
        if not base_url:
            raise ValueError("Base URL cannot be empty.")

        values["base_url"] = urljoin(base_url.rstrip("/") + "/", "")
        return values

    def base_url_with_endpoint(self, endpoint: str = "") -> str:
        """
        Construct the full URL for a given endpoint.

        :param endpoint: API endpoint.
        :return: Full URL as a string.
        """
        return f"{urljoin(self.base_url.rstrip('/') + '/', '')}{endpoint.lstrip('/')}"

    def get_client(self) -> HttpClient:
        """
        Return or create the HTTP client instance.

        :return: Configured `HttpClient`.
        :raises P2PException: If the existing client is not an instance of HttpClient.
        """
        if not self.p2p_client:
            self.p2p_client = HttpClient(
                base_url=str(self.base_url), timeout=self.timeout, logger=self.p2p_logger, headers=self.headers
            )

        return self.p2p_client

    def logger(self) -> logging.Logger:
        """
        Configure and return the logger.
        """
        if not self.p2p_logger:
            self.p2p_logger = self._create_logger()
        return self.p2p_logger

    def _create_logger(self) -> logging.Logger:
        """
        Create and configure a logger based on logger_config or a default setup.

        :return: Configured logger instance.
        """
        logger = logging.getLogger("P2P Checkout Logger")
        logger.setLevel(logging.DEBUG)

        if not logger.handlers:
            handler = logging.StreamHandler()
            formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
            handler.setFormatter(formatter)
            logger.addHandler(handler)

        if self.loggerConfig:
            logger.setLevel(self.loggerConfig.get("level", logging.DEBUG))
            custom_formatter = self.loggerConfig.get("formatter")
            if custom_formatter:
                logger.handlers[0].setFormatter(logging.Formatter(custom_formatter))

        return logger

    def authentication(self) -> Authentication:
        """
        Return an `Authentication` instance.
        """
        auth = Authentication(
            {
                "login": self.login,
                "tranKey": self.tranKey,
                "auth_additional": self.auth_additional,
            }
        )
        return auth

    def carrier(self) -> Carrier:
        """
        Return or create the carrier instance.
        """
        from checkout.clients.rest_client import RestCarrier

        if not isinstance(self._carrier_instance, RestCarrier):
            self._carrier_instance = RestCarrier(self)
        return self._carrier_instance



import unittest

from pydantic import ValidationError

from checkout.entities.tax_detail import TaxDetail


class TaxtDetailTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test the initialization of a TaxDetail object.
        """
        tax_detail = TaxDetail(kind="VAT", amount=100.0, base=500.0)

        self.assertEqual(tax_detail.kind, "VAT")
        self.assertEqual(tax_detail.amount, 100.0)
        self.assertEqual(tax_detail.base, 500.0)

    def test_to_dict(self):
        """
        Test the to_dict method of TaxDetail.
        """
        tax_detail = TaxDetail(kind="VAT", amount=100.0, base=500.0)

        expected_dict = {
            "kind": "VAT",
            "amount": 100.0,
            "base": 500.0,
        }

        self.assertEqual(tax_detail.to_dict(), expected_dict)

    def test_to_dict_without_base(self):
        """
        Test the to_dict method when base is not provided.
        """
        tax_detail = TaxDetail(kind="VAT", amount=100.0)

        expected_dict = {"amount": 100, "base": None, "kind": "VAT"}

        self.assertEqual(tax_detail.to_dict(), expected_dict)

    def test_missing_required_fields(self):
        """
        Test that missing required fields raise a validation error.
        """
        with self.assertRaises(ValidationError):
            TaxDetail(amount=100.0)



import unittest

from checkout.entities.amount import Amount
from checkout.entities.dispersion_payment import DispersionPayment
from checkout.entities.payment import Payment


class DispersionPaymentTest(unittest.TestCase):

    def test_dispersion_payment_initialization(self):

        data = {
            "reference": "REF001",
            "description": "Main Payment",
            "amount": {"currency": "USD", "total": 100.0},
            "dispersion": [
                Payment(reference="DISP001", description="Split Payment 1", amount=Amount(total=1000, currency="COP")),
                {
                    "reference": "DISP002",
                    "description": "Split Payment 2",
                    "amount": {"currency": "COP", "total": 1000},
                },
            ],
        }

        dispersion_payment = DispersionPayment(**data)

        self.assertEqual(dispersion_payment.reference, "REF001")
        self.assertEqual(dispersion_payment.description, "Main Payment")
        self.assertEqual(len(dispersion_payment.dispersion), 2)

        self.assertEqual(dispersion_payment.dispersion[0].reference, "DISP001")
        self.assertEqual(dispersion_payment.dispersion[0].description, "Split Payment 1")
        self.assertEqual(dispersion_payment.dispersion[0].amount.total, 1000.0)

        self.assertEqual(dispersion_payment.dispersion[1].reference, "DISP002")
        self.assertEqual(dispersion_payment.dispersion[1].description, "Split Payment 2")
        self.assertEqual(dispersion_payment.dispersion[1].amount.total, 1000.0)

    def test_set_dispersion(self):
        """
        Test setting dispersion payments after initialization.
        """
        data = {
            "reference": "REF001",
            "description": "Main Payment",
            "amount": {"currency": "USD", "total": 100.0},
        }

        dispersion_payment = DispersionPayment(**data)

        dispersion_data = [
            {"reference": "DISP001", "description": "Split Payment 1", "amount": {"currency": "USD", "total": 50.0}},
            {"reference": "DISP002", "description": "Split Payment 2", "amount": {"currency": "USD", "total": 50.0}},
        ]

        dispersion_payment.set_dispersion(dispersion_data)

        self.assertEqual(len(dispersion_payment.dispersion), 2)
        self.assertEqual(dispersion_payment.dispersion[0].reference, "DISP001")
        self.assertEqual(dispersion_payment.dispersion[1].reference, "DISP002")

    def test_dispersion_to_dict(self):
        """
        Test converting the dispersion payments to a dictionary.
        """
        data = {
            "reference": "REF001",
            "description": "Main Payment",
            "amount": {"currency": "USD", "total": 100.0},
            "dispersion": [
                {
                    "reference": "DISP001",
                    "description": "Split Payment 1",
                    "amount": {"currency": "USD", "total": 50.0},
                },
                {
                    "reference": "DISP002",
                    "description": "Split Payment 2",
                    "amount": {"currency": "USD", "total": 50.0},
                },
            ],
        }

        dispersion_payment = DispersionPayment(**data)
        print(dispersion_payment.dispersion_to_dict())

        expected_dispersion = [
            {
                "reference": "DISP001",
                "description": "Split Payment 1",
                "amount": {
                    "currency": "USD",
                    "total": 50.0,
                    "taxes": [],
                    "details": [],
                    "tip": None,
                    "insurance": None,
                },
                "allowPartial": False,
                "shipping": None,
                "items": [],
                "recurring": None,
                "discount": None,
                "subscribe": False,
                "agreement": None,
                "agreementType": "",
                "modifiers": [],
                "fields": [],
            },
            {
                "reference": "DISP002",
                "description": "Split Payment 2",
                "amount": {
                    "currency": "USD",
                    "total": 50.0,
                    "taxes": [],
                    "details": [],
                    "tip": None,
                    "insurance": None,
                },
                "allowPartial": False,
                "shipping": None,
                "items": [],
                "recurring": None,
                "discount": None,
                "subscribe": False,
                "agreement": None,
                "agreementType": "",
                "modifiers": [],
                "fields": [],
            },
        ]

        self.assertEqual(dispersion_payment.dispersion_to_dict(), expected_dispersion)

    def test_dispersion_payment_to_dict(self):
        """
        Test converting the entire DispersionPayment object to a dictionary.
        """
        data = {
            "reference": "REF001",
            "description": "Main Payment",
            "amount": {"currency": "USD", "total": 100.0},
            "dispersion": [
                {
                    "reference": "DISP001",
                    "description": "Split Payment 1",
                    "amount": {"currency": "USD", "total": 50.0},
                },
                {
                    "reference": "DISP002",
                    "description": "Split Payment 2",
                    "amount": {"currency": "USD", "total": 50.0},
                },
            ],
        }

        dispersion_payment = DispersionPayment(**data)

        expected_dict = {
            "reference": "REF001",
            "description": "Main Payment",
            "amount": {"currency": "USD", "total": 100.0, "taxes": [], "details": [], "tip": None, "insurance": None},
            "allowPartial": False,
            "shipping": None,
            "items": [],
            "recurring": None,
            "discount": None,
            "subscribe": False,
            "agreement": None,
            "agreementType": "",
            "modifiers": [],
            "fields": [],
            "dispersion": [
                {
                    "reference": "DISP001",
                    "description": "Split Payment 1",
                    "amount": {
                        "currency": "USD",
                        "total": 50.0,
                        "taxes": [],
                        "details": [],
                        "tip": None,
                        "insurance": None,
                    },
                    "allowPartial": False,
                    "shipping": None,
                    "items": [],
                    "recurring": None,
                    "discount": None,
                    "subscribe": False,
                    "agreement": None,
                    "agreementType": "",
                    "modifiers": [],
                    "fields": [],
                },
                {
                    "reference": "DISP002",
                    "description": "Split Payment 2",
                    "amount": {
                        "currency": "USD",
                        "total": 50.0,
                        "taxes": [],
                        "details": [],
                        "tip": None,
                        "insurance": None,
                    },
                    "allowPartial": False,
                    "shipping": None,
                    "items": [],
                    "recurring": None,
                    "discount": None,
                    "subscribe": False,
                    "agreement": None,
                    "agreementType": "",
                    "modifiers": [],
                    "fields": [],
                },
            ],
        }
        self.assertEqual(dispersion_payment.to_dict(), expected_dict)



import unittest

from checkout.entities.status import Status
from checkout.enums.status_enum import StatusEnum
from checkout.messages.responses.redirect import RedirectResponse


class RedirectResponseTest(unittest.TestCase):

    def test_initialization_with_all_fields(self):
        """
        Test the initialization of RedirectResponse with all fields provided.
        """
        status = Status(status=StatusEnum.OK, reason="Approved")
        redirect_response = RedirectResponse(
            requestId="12345",
            process_url="https://example.com/session",
            status=status,
        )

        self.assertEqual(redirect_response.request_id, "12345")
        self.assertEqual(redirect_response.process_url, "https://example.com/session")
        self.assertIsInstance(redirect_response.status, Status)
        self.assertTrue(redirect_response.is_successful())

    def test_initialization_with_partial_fields(self):
        """
        Test the initialization of RedirectResponse with only required fields.
        """
        status = Status(status=StatusEnum.REJECTED, reason="Declined")
        redirect_response = RedirectResponse(
            requestId="67890",
            process_url="https://example.com/session",
            status=status,
        )

        self.assertEqual(redirect_response.request_id, "67890")
        self.assertEqual(redirect_response.process_url, "https://example.com/session")
        self.assertFalse(redirect_response.is_successful())

    def test_to_dict(self):
        """
        Test the to_dict method for RedirectResponse.
        """
        status = Status(status=StatusEnum.OK, reason="Approved")
        redirect_response = RedirectResponse(
            requestId="12345",
            process_url="https://example.com/session",
            status=status,
        )

        expected_dict = {
            "requestId": "12345",
            "processUrl": "https://example.com/session",
            "status": status.to_dict(),
        }

        self.assertEqual(redirect_response.to_dict(), expected_dict)

    def test_is_successful_with_no_status(self):
        """
        Test is_successful method when status is None.
        """
        redirect_response = RedirectResponse(
            requestId="12345",
            process_url="https://example.com/session",
            status=None,
        )

        self.assertFalse(redirect_response.is_successful())



from checkout.checkout import Checkout
from checkout.messages.requests.collect import CollectRequest
from checkout.messages.requests.redirect import RedirectRequest

__all__ = ["Checkout", "RedirectRequest", "CollectRequest"]



import unittest

from checkout.messages.requests.redirect import RedirectRequest


class RedirectRequestTest(unittest.TestCase):
    def test_initialization_with_minimal_data(self):
        """
        Test initialization of RedirectRequest with required fields only.
        """
        data = {"returnUrl": "https://example.com/return", "ipAddress": "192.168.1.1", "userAgent": "Test User Agent"}
        redirect_request = RedirectRequest(**data)

        self.assertEqual(redirect_request.return_url, "https://example.com/return")
        self.assertEqual(redirect_request.ip_address, "192.168.1.1")
        self.assertEqual(redirect_request.user_agent, "Test User Agent")
        self.assertEqual(redirect_request.locale, "es_CO")

    def test_initialization_with_all_fields(self):
        """
        Test initialization of RedirectRequest with all fields.
        """
        data = {
            "locale": "en_US",
            "payer": {"document": "123456789", "name": "John", "surname": "Doe"},
            "buyer": {"document": "987654321", "name": "Jane", "surname": "Doe"},
            "payment": {"reference": "TEST_REF"},
            "subscription": {"reference": "SUB123", "description": "Test Subscription"},
            "returnUrl": "https://example.com/return",
            "paymentMethod": "credit_card",
            "cancelUrl": "https://example.com/cancel",
            "ipAddress": "192.168.1.1",
            "userAgent": "Test User Agent",
            "expiration": "2023-12-31T23:59:59Z",
            "captureAddress": "true",
            "skipResult": "true",
            "noBuyerFill": "true",
        }
        redirect_request = RedirectRequest(**data)

        self.assertEqual(redirect_request.locale, "en_US")
        self.assertEqual(redirect_request.return_url, "https://example.com/return")
        self.assertEqual(redirect_request.payment_method, "credit_card")
        self.assertEqual(redirect_request.cancel_url, "https://example.com/cancel")
        self.assertEqual(redirect_request.ip_address, "192.168.1.1")
        self.assertEqual(redirect_request.user_agent, "Test User Agent")
        self.assertEqual(redirect_request.expiration, "2023-12-31T23:59:59Z")
        self.assertTrue(redirect_request.capture_address)
        self.assertTrue(redirect_request.skip_result)
        self.assertTrue(redirect_request.no_buyer_fill)

    def test_from_dict(self):
        """
        Test creating RedirectRequest from a dictionary.
        """
        data = {"returnUrl": "https://example.com/return", "ipAddress": "192.168.1.1", "userAgent": "Test User Agent"}
        redirect_request = RedirectRequest.model_validate(data)

        self.assertIsInstance(redirect_request, RedirectRequest)
        self.assertEqual(redirect_request.return_url, "https://example.com/return")
        self.assertEqual(redirect_request.ip_address, "192.168.1.1")
        self.assertEqual(redirect_request.user_agent, "Test User Agent")

    def test_to_dict(self):
        """
        Test converting RedirectRequest to a dictionary.
        """
        data = {
            "returnUrl": "https://example.com/return",
            "ipAddress": "192.168.1.1",
            "userAgent": "Test User Agent",
            "captureAddress": True,
        }
        redirect_request = RedirectRequest(**data)
        result = redirect_request.to_dict()

        expected = {
            "locale": "es_CO",
            "returnUrl": "https://example.com/return",
            "ipAddress": "192.168.1.1",
            "userAgent": "Test User Agent",
            "captureAddress": "true",
            "skipResult": "false",
            "noBuyerFill": "false",
        }
        self.assertEqual(result, expected)

    def test_optional_fields_defaults(self):
        """
        Test that optional fields have correct default values.
        """
        data = {"returnUrl": "https://example.com/return", "ipAddress": "192.168.1.1", "userAgent": "Test User Agent"}
        redirect_request = RedirectRequest(**data)

        self.assertIsNone(redirect_request.payer)
        self.assertIsNone(redirect_request.buyer)
        self.assertIsNone(redirect_request.payment)
        self.assertIsNone(redirect_request.subscription)
        self.assertIsNone(redirect_request.expiration)
        self.assertIsNone(redirect_request.payment_method)
        self.assertIsNone(redirect_request.cancel_url)



from typing import Protocol

from checkout.messages.requests.collect import CollectRequest
from checkout.messages.requests.redirect import RedirectRequest
from checkout.messages.responses.information import InformationResponse
from checkout.messages.responses.redirect import RedirectResponse
from checkout.messages.responses.reverse import ReverseResponse


class Carrier(Protocol):
    def request(self, redirect_request: RedirectRequest) -> RedirectResponse:
        """
        Handle a redirect request.
        """

    def query(self, request_id: str) -> InformationResponse:
        """
        Query a redirect by request ID.
        """

    def collect(self, collect_request: CollectRequest) -> InformationResponse:
        """
        Collect redirect information from a request.
        """

    def reverse(self, transaction_id: str) -> ReverseResponse:
        """
        Reverse a transaction by its ID.
        """



import unittest

from checkout.entities.address import Address
from checkout.entities.person import Person


class PersonTest(unittest.TestCase):

    def test_person_initialization_with_defaults(self):
        """
        Test initialization with default values.
        """
        person = Person()

        assert person.document == ""
        assert person.documentType == ""
        assert person.name == ""
        assert person.surname == ""
        assert person.company == ""
        assert person.email == ""
        assert person.mobile == ""
        assert person.address is None

    def test_person_initialization_with_values(self):
        """
        Test initialization with provided values.
        """
        address = Address(
            street="123 Main St", city="Testville", state="TX", postal_code="12345", country="US", phone="123-456-7890"
        )

        person = Person(
            document="123456789",
            documentType="TIN",
            name="John",
            surname="Doe",
            company="TestCorp",
            email="john.doe@example.com",
            mobile="1234567890",
            address=address,
        )

        assert person.document == "123456789"
        assert person.documentType == "TIN"
        assert person.name == "John"
        assert person.surname == "Doe"
        assert person.company == "TestCorp"
        assert person.email == "john.doe@example.com"
        assert person.mobile == "1234567890"
        assert person.address == address

    def test_person_is_business(self):
        """
        Test the `is_business` method.
        """
        person_business = Person(documentType="TIN")
        person_non_business = Person(documentType="ID")

        assert person_business.is_business() is True
        assert person_non_business.is_business() is False

    def test_person_to_dict(self):
        """
        Test the `to_dict` method.
        """
        address = Address(
            street="123 Main St", city="Testville", state="TX", postal_code="12345", country="US", phone="123-456-7890"
        )

        person = Person(
            document="123456789",
            documentType="TIN",
            name="John",
            surname="Doe",
            company="TestCorp",
            email="john.doe@example.com",
            mobile="1234567890",
            address=address,
        )

        person_dict = person.to_dict()

        assert person_dict["document"] == "123456789"
        assert person_dict["documentType"] == "TIN"
        assert person_dict["name"] == "John"
        assert person_dict["surname"] == "Doe"
        assert person_dict["company"] == "TestCorp"
        assert person_dict["email"] == "john.doe@example.com"
        assert person_dict["mobile"] == "1234567890"
        assert person_dict["address"]["street"] == "123 Main St"

    def test_person_to_dict_without_address(self):
        """
        Test the `to_dict` method when address is not set.
        """
        person = Person(
            document="123456789",
            documentType="TIN",
            name="John",
            surname="Doe",
            company="TestCorp",
            email="john.doe@example.com",
            mobile="1234567890",
        )

        person_dict = person.to_dict()

        assert person_dict["document"] == "123456789"
        assert person_dict["documentType"] == "TIN"
        assert person_dict["name"] == "John"
        assert person_dict["surname"] == "Doe"
        assert person_dict["company"] == "TestCorp"
        assert person_dict["email"] == "john.doe@example.com"
        assert person_dict["mobile"] == "1234567890"
        assert "address" not in person_dict



import unittest

from checkout.entities.name_value_pair import NameValuePair
from checkout.mixins.fields_mixin import FieldsMixin


class FieldsMixinTest(unittest.TestCase):

    def test_get_fields(self):
        """
        Test retrieving the fields list.
        """
        mixin = FieldsMixin()
        mixin.fields = [NameValuePair(keyword="key1", value="value1")]
        fields = mixin.get_fields()
        self.assertEqual(len(fields), 1)
        self.assertEqual(fields[0].keyword, "key1")
        self.assertEqual(fields[0].value, "value1")

    def test_set_fields_with_dict(self):
        """
        Test setting fields with a dictionary containing an 'item' key.
        """
        mixin = FieldsMixin()
        fields_data = {"item": [{"keyword": "key1", "value": "value1"}, {"keyword": "key2", "value": "value2"}]}
        mixin.set_fields(fields_data)
        self.assertEqual(len(mixin.fields), 2)
        self.assertEqual(mixin.fields[0].keyword, "key1")
        self.assertEqual(mixin.fields[1].keyword, "key2")

    def test_set_fields_with_list(self):
        """
        Test setting fields with a list of dictionaries.
        """
        mixin = FieldsMixin()
        fields_data = [{"keyword": "key1", "value": "value1"}, {"keyword": "key2", "value": "value2"}]
        mixin.set_fields(fields_data)
        self.assertEqual(len(mixin.fields), 2)
        self.assertEqual(mixin.fields[0].keyword, "key1")
        self.assertEqual(mixin.fields[1].value, "value2")

    def test_fields_to_array(self):
        """
        Test converting fields to a list of dictionaries.
        """
        mixin = FieldsMixin()
        mixin.fields = [
            NameValuePair(keyword="key1", value="value1"),
            NameValuePair(keyword="key2", value="value2"),
        ]
        result = mixin.fields_to_array()
        expected = [
            {"keyword": "key1", "value": "value1", "displayOn": "none"},
            {"keyword": "key2", "value": "value2", "displayOn": "none"},
        ]
        self.assertEqual(result, expected)

    def test_fields_to_key_value(self):
        """
        Test converting fields to a key-value dictionary.
        """
        mixin = FieldsMixin()
        mixin.fields = [
            NameValuePair(keyword="key1", value="value1"),
            NameValuePair(keyword="key2", value="value2"),
        ]
        result = mixin.fields_to_key_value()
        expected = {"key1": "value1", "key2": "value2"}
        self.assertEqual(result, expected)

    def test_add_field_with_dict(self):
        """
        Test adding a field using a dictionary.
        """
        mixin = FieldsMixin()
        field_data = {"keyword": "key1", "value": "value1"}
        mixin.add_field(field_data)
        self.assertEqual(len(mixin.fields), 1)
        self.assertEqual(mixin.fields[0].keyword, "key1")
        self.assertEqual(mixin.fields[0].value, "value1")

    def test_add_field_with_name_value_pair(self):
        """
        Test adding a field using a NameValuePair instance.
        """
        mixin = FieldsMixin()
        nvp = NameValuePair(keyword="key1", value="value1")
        mixin.add_field(nvp)
        self.assertEqual(len(mixin.fields), 2)
        self.assertEqual(mixin.fields[0].keyword, "key1")
        self.assertEqual(mixin.fields[0].value, "value1")

    def test_set_fields_with_item_key(self):
        """
        Test setting fields when the input is a dictionary with an 'item' key.
        """
        fields_data = {"item": [{"keyword": "key1", "value": "value1"}, {"keyword": "key2", "value": "value2"}]}

        mixin = FieldsMixin()
        mixin.set_fields(fields_data)
        self.assertEqual(len(mixin.fields), 2)

        self.assertEqual(mixin.fields[0].keyword, "key1")
        self.assertEqual(mixin.fields[1].keyword, "key2")

    def test_fields_to_key_value_with_non_name_value_pair(self):
        """
        Test fields_to_key_value skips non-NameValuePair objects.
        """
        mixin = FieldsMixin()
        mixin.fields = [NameValuePair(keyword="key1", value="value1"), "invalid_field"]
        result = mixin.fields_to_key_value()
        expected = {"key1": "value1"}
        self.assertEqual(result, expected)



from typing import Any, Dict, Type, TypeVar, Union

from checkout.contracts.carrier import Carrier
from checkout.entities.settings import Settings
from checkout.exceptions.checkout_exception import CheckoutException
from checkout.messages.requests.collect import CollectRequest
from checkout.messages.requests.redirect import RedirectRequest
from checkout.messages.responses.information import InformationResponse
from checkout.messages.responses.redirect import RedirectResponse
from checkout.messages.responses.reverse import ReverseResponse

T = TypeVar("T", RedirectRequest, CollectRequest)


class Checkout:
    """
    Main class for interacting with PlaceToPay.
    """

    def __init__(self, data: Dict[str, Any]) -> None:
        """
        Initialize the PlacetoPay instance with the provided settings.

        :param data: Configuration dictionary for settings.
        """
        self.settings: Settings = Settings(**data)
        self.logger = self.settings.logger()

    def _validate_request(self, request: Union[RedirectRequest, CollectRequest, Dict], expected_class: Type[T]) -> T:
        """
        Validate the request object and convert it to the expected class if necessary.

        :param request: The request object or dictionary.
        :param expected_class: The expected class type for the request.
        :return: A validated request object.
        :raises P2PException: If the request is invalid.
        """
        if isinstance(request, dict):
            try:
                request = expected_class(**request)
            except Exception as e:
                self.logger.error(f"Failed to convert dictionary to {expected_class.__name__}: {e}")
                raise CheckoutException(f"Failed to convert dictionary to {expected_class.__name__}: {e}")

        if not isinstance(request, expected_class):
            self.logger.error(f"Invalid request type: {type(request).__name__}. Expected {expected_class.__name__}.")
            raise CheckoutException(
                f"Invalid request type: {type(request).__name__}. Expected {expected_class.__name__}."
            )

        self.logger.debug(f"Request validated as {expected_class.__name__}.")
        return request

    @property
    def carrier(self) -> Carrier:
        """
        Access the carrier instance from settings.
        """
        return self.settings.carrier()

    def request(self, redirect_request: Union[RedirectRequest, Dict]) -> RedirectResponse:
        """
        Handle a redirect request.

        :param redirect_request: RedirectRequest instance or dictionary with request data.
        :return: RedirectResponse object.
        """
        redirect_request = self._validate_request(redirect_request, RedirectRequest)
        return self.carrier.request(redirect_request)

    def query(self, request_id: str) -> InformationResponse:
        """
        Query a session by request ID.

        :param request_id: The ID of the request to query.
        :return: Information object.
        """
        self.logger.info(f"Querying request ID: {request_id}.")
        return self.carrier.query(request_id)

    def collect(self, collect_request: Union[CollectRequest, Dict]) -> InformationResponse:
        """
        Handle a collect request.

        :param collect_request: CollectRequest instance or dictionary with request data.
        :return: Information object.
        """
        collect_request = self._validate_request(collect_request, CollectRequest)
        self.logger.info("Collect init")
        return self.carrier.collect(collect_request)

    def reverse(self, internal_reference: str) -> ReverseResponse:
        """
        Reverse a transaction.

        :param internal_reference: The internal reference of the transaction to reverse.
        :return: ReverseResponse object.
        """
        self.logger.info(f"Reversing transaction with reference: {internal_reference}.")
        return self.carrier.reverse(internal_reference)



from typing import Optional

from pydantic import ConfigDict, Field

from checkout.decorators.convert_to_boolean import convert_booleans_to_strings
from checkout.entities.instrument import Instrument
from checkout.messages.requests.redirect import RedirectRequest


class CollectRequest(RedirectRequest):

    model_config = ConfigDict(populate_by_name=True, arbitrary_types_allowed=True)

    instrument: Optional[Instrument] = Field(..., description="Instrument details")
    return_url: str = Field(default="", alias="returnUrl", description="URL to return to after processing")
    ip_address: str = Field(default="", alias="ipAddress", description="IP address of the user")
    user_agent: str = Field(default="", alias="userAgent", description="User agent of the user's browser")

    @convert_booleans_to_strings
    def to_dict(self) -> dict:
        """
        Convert the CollectRequest object to a dictionary.
        """
        parent_dict = super().to_dict()

        return {
            **parent_dict,
            "instrument": self.instrument.to_dict() if self.instrument else "",
        }



from typing import Optional

from pydantic import BaseModel, Field

from checkout.entities.status import Status


class Account(BaseModel):
    bankCode: str = Field(..., description="Code of the bank")
    bankName: str = Field(..., description="Name of the bank")
    accountType: str = Field(default="", description="Type of the account")
    accountNumber: str = Field(default="", description="Number of the account")
    status: Optional[Status] = Field(default=None, description="Status of the account")

    def to_dict(self) -> dict:
        """
        Convert the Account object to a dictionary.
        """
        return {
            "status": self.status.to_dict() if self.status else None,
            "bankCode": self.bankCode,
            "bankName": self.bankName,
            "accountType": self.accountType,
            "accountNumber": self.accountNumber,
        }

    def get_type(self) -> str:
        """
        Returns the type of the entity (always 'account').
        """
        return "account"

    def last_digits(self) -> str:
        """
        Returns the last 4 digits of the account number.
        """
        return self.accountNumber[-4:] if self.accountNumber else ""



import unittest

from checkout.entities.name_value_pair import NameValuePair
from checkout.enums.display_on_enum import DisplayOnEnum


class NameValuePairTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test initialization of NameValuePair with valid data.
        """
        pair = NameValuePair(keyword="testKey", value="testValue", displayOn=DisplayOnEnum.BOTH)
        self.assertEqual(pair.keyword, "testKey")
        self.assertEqual(pair.value, "testValue")
        self.assertEqual(pair.displayOn, DisplayOnEnum.BOTH)

    def test_default_values(self):
        """
        Test default values for optional fields.
        """
        pair = NameValuePair(keyword="defaultKey")
        self.assertEqual(pair.keyword, "defaultKey")
        self.assertIsNone(pair.value)
        self.assertEqual(pair.displayOn, DisplayOnEnum.NONE)

    def test_to_dict(self):
        """
        Test conversion of NameValuePair to a dictionary.
        """
        pair = NameValuePair(keyword="testKey", value={"key": "value"}, displayOn=DisplayOnEnum.RECEIPT)
        expected_dict = {
            "keyword": "testKey",
            "value": {"key": "value"},
            "displayOn": "receipt",
        }
        self.assertEqual(pair.to_dict(), expected_dict)

    def test_to_dict_exclude_none(self):
        """
        Test dictionary conversion with exclusion of None values.
        """
        pair = NameValuePair(keyword="testKey", displayOn=DisplayOnEnum.PAYMENT)
        expected_dict = {"keyword": "testKey", "displayOn": "payment", "value": None}
        self.assertEqual(pair.to_dict(), expected_dict)

    def test_value_as_list(self):
        """
        Test initialization when value is a list.
        """
        pair = NameValuePair(keyword="testKey", value=["item1", "item2"])
        self.assertEqual(pair.value, ["item1", "item2"])

    def test_value_as_dict(self):
        """
        Test initialization when value is a dictionary.
        """
        pair = NameValuePair(keyword="testKey", value={"key": "value"})
        self.assertEqual(pair.value, {"key": "value"})



import unittest

from checkout.entities.address import Address


class AddressTest(unittest.TestCase):

    def test_address_initialization(self):
        """
        Test the initialization of the Address object.
        """
        address = Address(
            street="123 Main St",
            city="Test City",
            state="Test State",
            postalCode="12345",
            country="Test Country",
            phone="123-456-7890",
        )

        assert address.street == "123 Main St"
        assert address.city == "Test City"
        assert address.state == "Test State"
        assert address.postalCode == "12345"
        assert address.country == "Test Country"
        assert address.phone == "123-456-7890"

    def test_address_to_dict(self):
        """
        Test the `to_dict` method.
        """
        address = Address(
            street="123 Main St",
            city="Test City",
            state="Test State",
            postalCode="12345",
            country="Test Country",
            phone="123-456-7890",
        )

        expected_dict = {
            "street": "123 Main St",
            "city": "Test City",
            "state": "Test State",
            "postalCode": "12345",
            "country": "Test Country",
            "phone": "123-456-7890",
        }

        assert address.to_dict() == expected_dict

    def test_address_to_dict_partial(self):
        """
        Test the `to_dict` method when only partial data is provided.
        """
        address = Address(street="123 Main St", city="Test City")

        expected_dict = {
            "street": "123 Main St",
            "city": "Test City",
            "state": "",
            "postalCode": "",
            "country": "",
            "phone": "",
        }

        assert address.to_dict() == expected_dict

    def test_address_default_values(self):
        """
        Test that the Address class initializes fields with default values when not provided.
        """
        address = Address()

        assert address.street == ""
        assert address.city == ""
        assert address.state == ""
        assert address.postalCode == ""
        assert address.country == ""
        assert address.phone == ""

    def test_address_empty_to_dict(self):
        """
        Test the `to_dict` method when all fields have default values.
        """
        address = Address()

        expected_dict = {"street": "", "city": "", "state": "", "postalCode": "", "country": "", "phone": ""}

        assert address.to_dict() == expected_dict



from typing import Dict, List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field

from checkout.entities.amount_conversion import AmountConversion
from checkout.entities.discount import Discount
from checkout.entities.name_value_pair import NameValuePair
from checkout.entities.status import Status
from checkout.enums.status_enum import StatusEnum


class Transaction(BaseModel):
    reference: str = Field(..., description="Commerce-provided reference")
    internal_reference: Union[str, int] = Field(
        default="", alias="internalReference", description="PlacetoPay reference"
    )
    payment_method: str = Field(default="", alias="paymentMethod", description="Payment method identifier")
    payment_method_name: str = Field(default="", alias="paymentMethodName", description="Payment method name")
    issuer_name: str = Field(default="", alias="issuerName", description="Name of the issuer")
    amount: Optional[AmountConversion] = Field(default=None, description="Amount conversion details")
    authorization: str = Field(default="", description="Authorization code")
    receipt: str = Field(default="", description="Receipt number")
    franchise: str = Field(default="", description="Franchise identifier")
    refunded: bool = Field(default=False, description="Refund status of the transaction")
    processor_fields: List[NameValuePair] = Field(
        default_factory=list, alias="processorFields", description="Processor fields"
    )
    discount: Optional[Discount] = Field(default=None, description="Discount information")
    status: Optional[Status] = Field(default=None, description="Transaction status")
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    def is_successful(self) -> bool:
        """
        Determines if the transaction is valid (query successful, not necessarily approved).
        """
        return bool(self.status and self.status.status != StatusEnum.ERROR)

    def is_approved(self) -> bool:
        """
        Determines if the transaction has been approved.
        """
        return bool(self.status and self.status.status == StatusEnum.APPROVED)

    def set_processor_fields(self, data: List[dict]) -> None:
        """
        Set the processor fields from a list of dictionaries.
        """
        if isinstance(data, dict) and "item" in data:
            data = data["item"]

        self.processor_fields = [NameValuePair(**nvp) for nvp in data]

    def processor_fields_to_array(self) -> List[dict]:
        """
        Convert processor fields to a list of dictionaries.
        """
        return [field.to_dict() for field in self.processor_fields]

    def additional_data(self) -> Dict[str, str]:
        """
        Parse processor fields as a key-value dictionary.
        """
        return {field.keyword: field.value for field in self.processor_fields}

    def to_dict(self) -> dict:
        """
        Convert the Transaction object to a dictionary.
        """
        return {
            "status": self.status.to_dict() if self.status else None,
            "internalReference": self.internal_reference,
            "paymentMethod": self.payment_method,
            "paymentMethodName": self.payment_method_name,
            "issuerName": self.issuer_name,
            "amount": self.amount.to_dict() if self.amount else None,
            "authorization": self.authorization,
            "reference": self.reference,
            "receipt": self.receipt,
            "franchise": self.franchise,
            "refunded": self.refunded,
            "discount": self.discount.to_dict() if self.discount else None,
            "processorFields": self.processor_fields_to_array(),
        }



import json
from typing import Any, Callable
from unittest.mock import Mock

import requests
from requests.models import Response


class RedirectResponseMock:
    @staticmethod
    def get_mock_response(file_name: str, status_code: int) -> Mock:
        """
        Load mock response data from a file and return a fully mocked Response object.

        :param file_name: Name of the file containing the mock response.
        :param status_code: HTTP status code to set for the response.
        :return: A mocked Response object.
        """
        file_path = f"checkout/tests/mocks/responses/{file_name}.json"
        with open(file_path, "r") as file:
            body = json.load(file)

        mock_response = Mock(spec=Response)
        mock_response.status_code = status_code
        mock_response.json.return_value = body
        mock_response.text = json.dumps(body)
        mock_response.headers = {
            200: {
                "Content-Type": "application/json",
                "X-Mock-Status": "Success",
            },
            400: {
                "Content-Type": "application/json",
                "X-Mock-Status": "Bad Request",
            },
            401: {
                "Content-Type": "application/json",
                "X-Mock-Status": "Unauthorized",
                "WWW-Authenticate": 'Bearer realm="Access to the staging site"',
            },
        }.get(status_code, {"Content-Type": "application/json"})

        if status_code >= 400:
            mock_response.raise_for_status.side_effect = requests.exceptions.HTTPError(
                f"{status_code} Client Error: Unauthorized for url",
                response=mock_response,
            )
        else:
            mock_response.raise_for_status = lambda: {}

        return mock_response

    @staticmethod
    def mock_response_decorator(file_name: str, status_code: int = 200) -> Callable:
        """
        A decorator that injects a mock response into the test function.

        :param file_name: Name of the file containing the mock response.
        :param status_code: HTTP status code to determine the response (default: 200).
        :return: A decorator function that modifies the test function to include the mock response.
        """

        def decorator(test_func: Callable) -> Callable:
            def wrapper(*args: Any, **kwargs: Any) -> Any:
                mock_response = RedirectResponseMock.get_mock_response(file_name, status_code)
                return test_func(*args, mock_response=mock_response, **kwargs)

            return wrapper

        return decorator



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field

from checkout.enums.status_enum import StatusEnum


class Status(BaseModel):
    status: StatusEnum
    reason: str
    message: str = Field(default="", description="Status message")
    date: Optional[str] = Field(default_factory=lambda: datetime.now().isoformat())

    def is_successful(self) -> bool:
        return self.status == StatusEnum.OK

    def is_approved(self) -> bool:
        return self.status == StatusEnum.APPROVED

    def is_rejected(self) -> bool:
        return self.status == StatusEnum.REJECTED

    def is_error(self) -> bool:
        return self.status == StatusEnum.ERROR

    @classmethod
    def quick(
        cls,
        status: StatusEnum,
        reason: str,
        message: str = "",
        date: Optional[str] = None,
    ) -> "Status":
        return cls(status=status, reason=reason, message=message, date=date)

    def to_dict(self) -> dict:
        """
        Convert the Status object to a dictionary.
        """
        return {
            "status": self.status.value,
            "reason": self.reason,
            "message": self.message,
            "date": self.date,
        }



from pydantic import BaseModel, Field


class AmountBase(BaseModel):
    currency: str = Field(default="COP", description="Currency code")
    total: float = Field(..., description="Total amount")

    def to_dict(self) -> dict:
        """
        Convert the AmountBase object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



from pydantic import BaseModel, Field


class Credit(BaseModel):
    code: str = Field(..., description="Credit code")
    type: str = Field(..., description="Type of credit")
    groupCode: str = Field(..., description="Group code of the credit")
    installment: int = Field(..., description="Number of installments when first created")

    def to_dict(self) -> dict:
        """
        Convert the Credit object to a dictionary using the Pydantic model_dump method.
        """
        return self.model_dump()



from typing import Dict

from checkout.contracts.carrier import Carrier
from checkout.entities.settings import Settings
from checkout.messages.requests.collect import CollectRequest
from checkout.messages.requests.redirect import RedirectRequest
from checkout.messages.responses.information import InformationResponse
from checkout.messages.responses.redirect import RedirectResponse
from checkout.messages.responses.reverse import ReverseResponse


class RestCarrier(Carrier):
    def __init__(self, settings: Settings):
        """
        Initialize the RestCarrier with the given settings.

        :param settings: Settings object with client, authentication, base URL, and logger.
        """
        self.settings = settings

    def _post(self, endpoint: str, arguments: Dict) -> Dict:
        """
        Make an HTTP POST request using the HttpClient.

        :param endpoint: API endpoint.
        :param arguments: Request data.
        :return: Response as a dictionary.
        """
        data = {**arguments, "auth": self.settings.authentication().to_dict()}
        return self.settings.get_client().post(endpoint=endpoint, json=data, headers=self.settings.headers)

    def request(self, redirect_request: RedirectRequest) -> RedirectResponse:
        """
        Handle a redirect request.
        """
        result = self._post("api/session", redirect_request.to_dict())
        return RedirectResponse(**result)

    def query(self, request_id: str) -> InformationResponse:
        """
        Query a session by request ID.
        """
        result = self._post(f"api/session/{request_id}", {})
        return InformationResponse(**result)

    def collect(self, collect_request: CollectRequest) -> InformationResponse:
        """
        Handle a collect request.
        """
        result = self._post("api/collect", collect_request.to_dict())
        return InformationResponse(**result)

    def reverse(self, transaction_id: str) -> ReverseResponse:
        """
        Reverse a transaction.
        """
        result = self._post("api/reverse", {"internalReference": transaction_id})
        return ReverseResponse(**result)



from typing import List, Optional, Union

from pydantic import Field

from checkout.entities.amount_base import AmountBase
from checkout.entities.amount_detail import AmountDetail
from checkout.entities.tax_detail import TaxDetail


class Amount(AmountBase):
    taxes: List[TaxDetail] = Field(default_factory=list, description="List of tax details")
    details: List[AmountDetail] = Field(default_factory=list, description="List of amount details")
    tip: Optional[float] = Field(default=None, description="Optional tip amount")
    insurance: Optional[float] = Field(default=None, description="Optional insurance amount")

    def set_taxes(self, taxes: List[Union[dict, TaxDetail]]) -> None:
        """
        Set the taxes for the amount object.
        """
        self.taxes = [TaxDetail(**tax) if isinstance(tax, dict) else tax for tax in taxes]

    def set_details(self, details: List[Union[dict, AmountDetail]]) -> None:
        """
        Set the details for the amount object.
        """
        self.details = []
        for detail in details:
            if isinstance(detail, dict):
                detail = AmountDetail(**detail)
            self.details.append(detail)

    def taxes_to_dict(self) -> List[dict]:
        """
        Convert taxes to a list of dictionaries.
        """
        return [tax.model_dump() for tax in self.taxes]

    def details_to_dict(self) -> List[dict]:
        """
        Convert details to a list of dictionaries.
        """
        return [detail.model_dump() for detail in self.details]

    def to_dict(self) -> dict:
        """
        Convert the Amount object to a dictionary including taxes and details.
        """
        parent_data = super().to_dict()
        return {
            **parent_data,
            "taxes": self.taxes_to_dict(),
            "details": self.details_to_dict(),
        }



from enum import Enum


class DisplayOnEnum(str, Enum):
    NONE = "none"
    """Not displayed in any view of the payment process. Ideal for keeping the data hidden."""

    PAYMENT = "payment"
    """Displayed only in the payment information entry view. Useful for showing data during input."""

    RECEIPT = "receipt"
    """Displayed only in the payment result view. Perfect for showing information in the final step."""

    BOTH = "both"
    """Displayed in both the payment and result views. Ensures visibility in all payment steps."""

    APPROVED = "approved"
    """Displayed in the result view only if the payment is successful."""



import base64
import datetime
import hashlib
import random
from typing import Dict

from checkout.exceptions.checkout_exception import CheckoutException


class Authentication:
    def __init__(self, config: Dict):
        """
        Initialize Authentication with the necessary configuration.

        :param config: Dictionary containing 'login' and 'tranKey'.
        """
        if "login" not in config or "tranKey" not in config:
            raise CheckoutException("No login or tranKey provided for authentication")

        self.login: str = config["login"]
        self.tran_key: str = config["tranKey"]

        self.raw_nonce = self._generate_raw_nonce()
        self.seed = self._generate_seed()
        self.tran_key_digest = self._generate_tran_key()

    def _generate_raw_nonce(self) -> bytes:
        """
        Generate a raw random nonce (16 bytes).

        :return: Raw nonce in bytes.
        """
        return random.getrandbits(128).to_bytes(16, byteorder="big")

    def _generate_seed(self) -> str:
        """
        Generate the seed (timestamp in ISO format).

        :return: ISO formatted timestamp.
        """
        return datetime.datetime.now(datetime.timezone.utc).isoformat()

    def _generate_tran_key(self) -> str:
        """
        Generate the tranKey by hashing the raw_nonce, seed, and tran_key using SHA256.

        :return: Base64 encoded tranKey string.
        """
        digest_input = self.raw_nonce + self.seed.encode("utf-8") + self.tran_key.encode("utf-8")
        digest = hashlib.sha256(digest_input).digest()
        return base64.b64encode(digest).decode("utf-8")

    def to_dict(self) -> Dict:
        """
        Return the authentication data as a dictionary.

        :return: Authentication dictionary.
        """
        return {
            "login": self.login,
            "tranKey": self.tran_key_digest,
            "nonce": base64.b64encode(self.raw_nonce).decode("utf-8"),
            "seed": self.seed,
        }



from typing import Optional

from pydantic import BaseModel, Field

from checkout.entities.status import Status
from checkout.entities.transaction import Transaction


class ReverseResponse(BaseModel):
    status: Optional[Status] = Field(default=None, description="Status of the reversal response")
    payment: Optional[Transaction] = Field(default=None, description="Transaction details associated with the reversal")

    def to_dict(self) -> dict:
        """
        Convert the ReverseResponse object to a dictionary.
        """
        return {
            "status": self.status.to_dict() if self.status else None,
            "payment": self.payment.to_dict() if self.payment else None,
        }



import json
import unittest
from unittest.mock import patch

from checkout.cases.redirect_response_mock import RedirectResponseMock
from checkout.checkout import Checkout
from checkout.entities.instrument import Instrument
from checkout.entities.token import Token
from checkout.exceptions.checkout_exception import CheckoutException
from checkout.exceptions.http_exceptions import ClientErrorException
from checkout.messages.requests.collect import CollectRequest
from checkout.messages.requests.redirect import RedirectRequest
from checkout.messages.responses.information import InformationResponse
from checkout.messages.responses.reverse import ReverseResponse


class CheckoutTest(unittest.TestCase):

    def setUp(self):
        """Set up shared test data."""
        self.settings_data = {
            "base_url": "https://example.com",
            "login": "test_login",
            "tranKey": "test_tranKey",
            "timeout": 10,
        }
        self.checkout = Checkout(self.settings_data)

    def test_initialization(self):
        """Test if the Checkout initializes correctly."""
        self.assertIsNotNone(self.checkout.settings)
        self.assertEqual(str(self.checkout.settings.base_url), "https://example.com/")

    def test_validate_request_invalid(self):
        """Test _validate_request with an invalid type."""
        with self.assertRaises(CheckoutException) as context:
            self.checkout._validate_request([], RedirectRequest)

        self.assertIn("Invalid request type: list. Expected RedirectRequest.", str(context.exception))

    def test_validate_request_invalid_dict_conversion(self):
        """
        Test _validate_request with a dictionary that cannot be converted to the expected class.
        """
        invalid_dict = {"invalid_field": "value"}
        with self.assertRaises(CheckoutException) as context:
            self.checkout._validate_request(invalid_dict, RedirectRequest)

        self.assertIn("Failed to convert dictionary to RedirectRequest", str(context.exception))

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("information_subscription_response_successful", 200)
    def test_query_subscription_valid(self, mock_post, mock_response):
        """
        Test the query method with a valid request ID and mock response.
        """
        mock_post.return_value = mock_response

        information_response = self.checkout.query("88860")

        self.assertIsInstance(information_response, InformationResponse)
        self.assertEqual(information_response.request_id, 88860)
        self.assertEqual(information_response.status.status, "APPROVED")
        self.assertEqual(information_response.status.reason, "00")
        self.assertEqual(information_response.status.message, "The request has been successfully approved")
        self.assertIsNotNone(information_response.status.date)

        self.assertIsNotNone(information_response.request)
        self.assertEqual(information_response.request.locale, "en_US")
        self.assertEqual(information_response.request.payer.document, "118877455")
        self.assertEqual(information_response.request.payer.name, "John")
        self.assertEqual(information_response.request.return_url, "https://www.google.com")

        self.assertIsNotNone(information_response.subscription)
        self.assertEqual(information_response.subscription.type, "token")
        self.assertEqual(information_response.subscription.status.status, "OK")
        self.assertEqual(information_response.subscription.status.reason, "00")
        self.assertEqual(
            information_response.subscription.status.message, "The request has been successfully processed"
        )

        self.assertIsNotNone(information_response.subscription.instrument)
        self.assertEqual(len(information_response.subscription.instrument), 8)
        self.assertEqual(information_response.subscription.instrument[0].keyword, "token")
        self.assertEqual(
            information_response.subscription.instrument[0].value,
            "71f293122c1ed577974f2249c9449c648d8dcb104cb531f2c77e3b6c8910aca0",
        )
        self.assertEqual(information_response.subscription.instrument[1].keyword, "subtoken")
        self.assertEqual(information_response.subscription.instrument[1].value, "2964322564071111")

        self.assertIsNone(information_response.payment)

        self.assertIsNone(information_response.last_transaction())
        self.assertIsNone(information_response.last_approved_transaction())
        self.assertEqual(information_response.last_authorization(), "")

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("information_payment_response_successful", 200)
    def test_query_payment_valid(self, mock_post, mock_response):
        """
        Test the query method with a valid request ID and mock response.
        """
        mock_post.return_value = mock_response

        information_response = self.checkout.query("88867")

        self.assertIsInstance(information_response, InformationResponse)
        self.assertEqual(information_response.request_id, 88867)
        self.assertEqual(information_response.status.status, "APPROVED")
        self.assertEqual(information_response.status.reason, "00")
        self.assertEqual(information_response.status.message, "The request has been successfully approved")
        self.assertIsNotNone(information_response.status.date)

        self.assertEqual(len(information_response.payment), 2)

        payment_1 = information_response.payment[0]
        self.assertEqual(payment_1.reference, "test_megapuntos_test_3")
        self.assertEqual(payment_1.authorization, "999999")
        self.assertEqual(payment_1.payment_method, "diners")
        self.assertEqual(payment_1.status.status, "APPROVED")
        self.assertEqual(payment_1.status.reason, "00")
        self.assertEqual(payment_1.amount.fromAmount.total, 10000)
        self.assertEqual(payment_1.amount.fromAmount.currency, "COP")
        self.assertEqual(payment_1.amount.toAmount.total, 2.24)
        self.assertEqual(payment_1.amount.toAmount.currency, "USD")

        payment_2 = information_response.payment[1]
        self.assertEqual(payment_2.reference, "test_megapuntos_test_3")
        self.assertEqual(payment_2.authorization, "000000")
        self.assertEqual(payment_2.payment_method, "master")
        self.assertEqual(payment_2.status.status, "REJECTED")
        self.assertEqual(payment_2.status.reason, "?2")

        self.assertEqual(payment_2.amount.fromAmount.total, 10000)
        self.assertEqual(payment_2.amount.fromAmount.currency, "COP")
        self.assertEqual(payment_2.amount.toAmount.total, 2178.45)
        self.assertEqual(payment_2.amount.toAmount.currency, "CLP")

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("collect_response_successful", 200)
    def test_collect_valid(self, mock_post, mock_response):
        """
        Test the collect method with a valid CollectRequest and mock response.
        """
        mock_post.return_value = mock_response

        token = Token(
            token="5caef08ecd1230088a12e8f7d9ce20e9134dc6fc049c8a4857c9ba6e942b16b2", subtoken="test_subtoken"
        )
        instrument = Instrument(token=token, pin="1234", password="secret")
        collect_request_data = {
            "instrument": instrument,
            "returnUrl": "https://checkout-co.placetopay.dev/home",
            "ipAddress": "186.86.52.226",
            "userAgent": "PostmanRuntime/7.42.0",
        }
        collect_request = CollectRequest.model_validate(collect_request_data)

        result = self.checkout.collect(collect_request)

        self.assertIsInstance(result, InformationResponse)
        self.assertEqual(result.request_id, 88866)
        self.assertEqual(result.status.status, "APPROVED")
        self.assertEqual(result.status.reason, "00")
        self.assertEqual(result.status.message, "La petición ha sido aprobada exitosamente")
        self.assertIsNotNone(result.status.date)

        self.assertEqual(len(result.payment), 1)

        payment = result.payment[0]
        self.assertEqual(payment.reference, "ref_collect_3")
        self.assertEqual(payment.authorization, "300159")
        self.assertEqual(payment.payment_method, "master")
        self.assertEqual(payment.status.status, "APPROVED")
        self.assertEqual(payment.status.message, "Aprobada")
        self.assertEqual(payment.amount.fromAmount.total, 10000)
        self.assertEqual(payment.amount.fromAmount.currency, "COP")
        self.assertEqual(payment.amount.toAmount.total, 2178.45)
        self.assertEqual(payment.amount.toAmount.currency, "CLP")

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("reverse_response_successful", 200)
    def test_reverse_valid(self, mock_post, mock_response):
        """
        Test the reverse method with a valid internal reference and mock response.
        """
        mock_post.return_value = mock_response

        internal_reference = "437987"
        result = self.checkout.reverse(internal_reference)

        self.assertIsInstance(result, ReverseResponse)
        self.assertIsNotNone(result.status)
        self.assertEqual(result.status.status, "APPROVED")
        self.assertEqual(result.status.reason, "00")
        self.assertEqual(result.status.message, "Aprobada")
        self.assertIsNotNone(result.status.date)

        self.assertIsNotNone(result.payment)
        payment = result.payment
        self.assertEqual(payment.reference, "ref_collect_3")
        self.assertEqual(payment.authorization, "300159")
        self.assertEqual(payment.payment_method, "master")
        self.assertEqual(payment.status.status, "APPROVED")
        self.assertEqual(payment.amount.fromAmount.total, 10000)
        self.assertEqual(payment.amount.fromAmount.currency, "COP")
        self.assertEqual(payment.amount.toAmount.total, 2178.45)
        self.assertEqual(payment.amount.toAmount.currency, "CLP")

        self.assertEqual(len(payment.processor_fields), 9)
        self.assertEqual(payment.processor_fields[0].keyword, "merchantCode")
        self.assertEqual(payment.processor_fields[0].value, "4549106521651")
        self.assertEqual(payment.processor_fields[1].keyword, "terminalNumber")
        self.assertEqual(payment.processor_fields[1].value, "98765432")
        self.assertEqual(payment.processor_fields[-1].value, "00")
        self.assertEqual(payment.processor_fields[-1].keyword, "b24")

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("redirect_response_fail_authentication", 401)
    def test_request_fails_bad_request(self, mock_post, mock_response):
        mock_post.return_value = mock_response

        redirect_request = {
            "returnUrl": "https://example.com/return",
            "ipAddress": "127.0.0.1",
            "userAgent": "Checkout Sandbox",
        }

        with self.assertRaises(ClientErrorException) as context:
            self.checkout.request(redirect_request)

        error_details = json.loads(str(context.exception))["error_details"]
        self.assertEqual("FAILED", error_details["status"]["status"])
        self.assertEqual(401, error_details["status"]["reason"])
        self.assertEqual("Failed authentication 101", error_details["status"]["message"])

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("collect_response_fails_token_not_valid", 400)
    def test_collect_fails_when_token_not_valid(self, mock_post, mock_response):

        token = Token(token="token_not_valid")
        instrument = Instrument(token=token, pin="1234", password="secret")
        collect_request_data = {
            "instrument": instrument,
            "returnUrl": "https://checkout-co.placetopay.dev/home",
            "ipAddress": "186.86.52.226",
            "userAgent": "PostmanRuntime/7.42.0",
        }

        mock_post.return_value = mock_response

        with self.assertRaises(ClientErrorException) as context:
            self.checkout.collect(CollectRequest.model_validate(collect_request_data))

        error_details = json.loads(str(context.exception))["error_details"]
        self.assertEqual("FAILED", error_details["status"]["status"])
        self.assertEqual("request_not_valid", error_details["status"]["reason"])
        self.assertEqual("La longitud del token no es correcta", error_details["status"]["message"])

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("information_fails_session_not_found", 401)
    def test_query_fails_when_session_not_found(self, mock_post, mock_response):
        mock_post.return_value = mock_response

        with self.assertRaises(ClientErrorException) as context:
            self.checkout.query("88608")

        error_details = json.loads(str(context.exception))["error_details"]
        self.assertEqual("FAILED", error_details["status"]["status"])
        self.assertEqual("unauthorized", error_details["status"]["reason"])
        self.assertEqual("La sesión no pertenece a su sitio", error_details["status"]["message"])

    @patch("requests.post")
    @RedirectResponseMock.mock_response_decorator("reverse_response_fails_transaction_not_found", 400)
    def test_reverse_fails_when_transaction_not_found(self, mock_post, mock_response):
        mock_post.return_value = mock_response

        with self.assertRaises(ClientErrorException) as context:
            self.checkout.reverse("123123123")

        error_details = json.loads(str(context.exception))["error_details"]
        self.assertEqual("FAILED", error_details["status"]["status"])
        self.assertEqual("request_not_valid", error_details["status"]["reason"])
        self.assertEqual("No existe la transacción que busca", error_details["status"]["message"])



from pydantic import BaseModel, Field


class Item(BaseModel):
    sku: str = Field(default="", description="Stock Keeping Unit (SKU) of the item")
    name: str = Field(default="", description="Name of the item")
    category: str = Field(default="", description="Category of the item")
    qty: str = Field(default="", description="Quantity of the item")
    price: str = Field(default="", description="Price of the item")
    tax: str = Field(default="", description="Tax applied to the item")

    def to_dict(self) -> dict:
        """
        Convert the Item object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



import unittest

from checkout.entities.credit import Credit


class TestCredit(unittest.TestCase):

    def test_credit_initialization(self):
        """
        Test that the Credit object initializes correctly with all fields.
        """
        credit = Credit(code="CREDIT123", type="Personal", groupCode="GRP001", installment=12)

        self.assertEqual(credit.code, "CREDIT123")
        self.assertEqual(credit.type, "Personal")
        self.assertEqual(credit.groupCode, "GRP001")
        self.assertEqual(credit.installment, 12)

    def test_credit_to_dict(self):
        """
        Test the `to_dict` method.
        """
        expected_dict = {
            "code": "CREDIT123",
            "type": "Personal",
            "groupCode": "GRP001",
            "installment": 12,
        }

        credit = Credit.model_validate(expected_dict)

        self.assertEqual(credit.to_dict(), expected_dict)

    def test_credit_missing_field(self):
        """
        Test that the Credit object raises a validation error when required fields are missing.
        """
        with self.assertRaises(ValueError):
            Credit(type="Personal", groupCode="GRP001", installment=12)

    def test_credit_field_types(self):
        """
        Test that the Credit object enforces the correct types for fields.
        """
        with self.assertRaises(ValueError):
            Credit(
                code="CREDIT123",
                type="Personal",
                groupCode="GRP001",
                installment="twelve",
            )



from typing import Any, Dict, Optional

from pydantic import BaseModel, Field


class PaymentModifier(BaseModel):
    TYPE_FEDERAL_GOVERNMENT: str = "FEDERAL_GOVERNMENT"

    type: Optional[str] = Field(default=None, description="Type of payment modifier")
    code: Optional[str] = Field(default=None, description="Code associated with the payment modifier")
    additional: Dict[str, Any] = Field(default_factory=dict, description="Additional data for the payment modifier")

    def set_type(self, type: Optional[str]) -> "PaymentModifier":
        """
        Set the type of the payment modifier.
        """
        self.type = type
        return self

    def set_code(self, code: Optional[str]) -> "PaymentModifier":
        """
        Set the code of the payment modifier.
        """
        self.code = code
        return self

    def get_additional(self, key: Optional[str] = None, default: Any = None) -> Any:
        """
        Get additional data by key. If key is not provided, return the entire additional data dictionary.
        """
        if key:
            return self.additional.get(key, default)
        return self.additional

    def set_additional(self, additional: Dict[str, Any]) -> "PaymentModifier":
        """
        Set the additional data for the payment modifier.
        """
        self.additional = additional
        return self

    def merge_additional(self, data: Dict[str, Any]) -> "PaymentModifier":
        """
        Merge the current additional data with new data.
        """
        self.additional.update(data)
        return self

    def to_dict(self) -> dict:
        """
        Convert the PaymentModifier object to a dictionary.
        """
        return {
            "type": self.type,
            "code": self.code,
            "additional": self.additional,
        }



from pydantic import BaseModel, Field


class AmountDetail(BaseModel):
    kind: str = Field(..., description="The type of amount")
    amount: float = Field(..., description="The amount value")

    def to_dict(self) -> dict:
        """
        Convert the AmountDetail object to a dictionary using the Pydantic `model_dump` method.
        """
        return self.model_dump()



import unittest

from checkout.entities.item import Item


class ItemTest(unittest.TestCase):

    def test_item_initialization(self):
        """
        Test the initialization of the Item class.
        """
        item = Item(sku="ITEM001", name="Test Item", category="Category A", qty="2", price="10.00", tax="1.20")

        self.assertEqual(item.sku, "ITEM001")
        self.assertEqual(item.name, "Test Item")
        self.assertEqual(item.category, "Category A")
        self.assertEqual(item.qty, "2")
        self.assertEqual(item.price, "10.00")
        self.assertEqual(item.tax, "1.20")

    def test_item_to_dict(self):
        """
        Test the `to_dict` method.
        """
        item = Item(sku="ITEM001", name="Test Item", category="Category A", qty="2", price="10.00", tax="1.20")

        expected_dict = {
            "sku": "ITEM001",
            "name": "Test Item",
            "category": "Category A",
            "qty": "2",
            "price": "10.00",
            "tax": "1.20",
        }

        self.assertEqual(item.to_dict(), expected_dict)

    def test_item_to_dict_with_defaults(self):
        """
        Test the `to_dict` method with default values.
        """
        item = Item()

        expected_dict = {
            "sku": "",
            "name": "",
            "category": "",
            "qty": "",
            "price": "",
            "tax": "",
        }

        self.assertEqual(item.to_dict(), expected_dict)

    def test_item_partial_initialization(self):
        """
        Test partial initialization of the Item class.
        """
        item = Item(sku="ITEM001", price="10.00")

        self.assertEqual(item.sku, "ITEM001")
        self.assertEqual(item.name, "")
        self.assertEqual(item.category, "")
        self.assertEqual(item.qty, "")
        self.assertEqual(item.price, "10.00")
        self.assertEqual(item.tax, "")



import unittest

from checkout.entities.account import Account
from checkout.entities.status import Status
from checkout.enums.status_enum import StatusEnum


class AccountTest(unittest.TestCase):

    def test_account_initialization(self):
        account = Account(
            bankCode="001",
            bankName="Test Bank",
            accountType="Savings",
            accountNumber="1234567890",
            status=Status(status=StatusEnum.OK, reason="Active"),
        )

        assert account.bankCode == "001"
        assert account.bankName == "Test Bank"
        assert account.accountType == "Savings"
        assert account.accountNumber == "1234567890"
        assert account.status.status == StatusEnum.OK
        assert account.status.reason == "Active"
        assert account.last_digits() == "7890"
        assert account.get_type() == "account"

    def test_account_to_dict(self):
        """
        Test the `to_dict` method.
        """
        status = Status(status=StatusEnum.OK, reason="Active", message="process ok")
        account = Account(
            bankCode="001", bankName="Test Bank", accountType="Savings", accountNumber="1234567890", status=status
        )

        expected_dict = {
            "status": status.to_dict(),
            "bankCode": "001",
            "bankName": "Test Bank",
            "accountType": "Savings",
            "accountNumber": "1234567890",
        }

        assert account.to_dict() == expected_dict

    def test_account_to_dict_without_status(self):
        """
        Test the `to_dict` method when status is not set.
        """
        account = Account(bankCode="001", bankName="Test Bank", accountType="Savings", accountNumber="1234567890")

        expected_dict = {
            "status": None,
            "bankCode": "001",
            "bankName": "Test Bank",
            "accountType": "Savings",
            "accountNumber": "1234567890",
        }

        assert account.to_dict() == expected_dict

    def test_account_get_type(self):
        """
        Test the `get_type` method.
        """
        account = Account(bankCode="001", bankName="Test Bank", accountType="Savings", accountNumber="1234567890")

        assert account.get_type() == "account"

    def test_account_last_digits(self):
        """
        Test the `last_digits` method.
        """
        account = Account(bankCode="001", bankName="Test Bank", accountType="Savings", accountNumber="1234567890")

        assert account.last_digits() == "7890"

    def test_account_last_digits_empty(self):
        """
        Test the `last_digits` method when account number is empty.
        """
        account = Account(bankCode="001", bankName="Test Bank", accountType="Savings", accountNumber="")

        assert account.last_digits() == ""



from datetime import datetime
import unittest

from checkout.entities.status import Status
from checkout.enums.status_enum import StatusEnum


class StatusTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test the initialization of a Status object.
        """
        status = Status(
            status=StatusEnum.OK, reason="All good", message="Operation successful", date="2024-11-25T12:00:00"
        )
        self.assertEqual(status.status, StatusEnum.OK)
        self.assertEqual(status.reason, "All good")
        self.assertEqual(status.message, "Operation successful")
        self.assertEqual(status.date, "2024-11-25T12:00:00")

    def test_default_date(self):
        """
        Test the default date generation.
        """
        status = Status(status=StatusEnum.OK, reason="All good")
        self.assertIsNotNone(status.date)
        self.assertTrue(datetime.fromisoformat(status.date))

    def test_is_successful(self):
        """
        Test the is_successful method.
        """
        status = Status(status=StatusEnum.OK, reason="Success")
        self.assertTrue(status.is_successful())

    def test_is_approved(self):
        """
        Test the is_approved method.
        """
        status = Status(status=StatusEnum.APPROVED, reason="Approved")
        self.assertTrue(status.is_approved())

    def test_is_rejected(self):
        """
        Test the is_rejected method.
        """
        status = Status(status=StatusEnum.REJECTED, reason="Rejected")
        self.assertTrue(status.is_rejected())

    def test_is_error(self):
        """
        Test the is_error method.
        """
        status = Status(status=StatusEnum.ERROR, reason="Error occurred")
        self.assertTrue(status.is_error())

    def test_quick_method(self):
        """
        Test the quick class method.
        """
        quick_status = Status.quick(
            status=StatusEnum.OK, reason="Quick init", message="Quick message", date="2024-11-25T12:00:00"
        )
        self.assertIsInstance(quick_status, Status)
        self.assertEqual(quick_status.status, StatusEnum.OK)
        self.assertEqual(quick_status.reason, "Quick init")
        self.assertEqual(quick_status.message, "Quick message")
        self.assertEqual(quick_status.date, "2024-11-25T12:00:00")

    def test_to_dict(self):
        """
        Test the to_dict method.
        """
        status = Status(
            status=StatusEnum.OK, reason="All good", message="Everything is fine", date="2024-11-25T12:00:00"
        )
        expected_dict = {
            "status": StatusEnum.OK.value,
            "reason": "All good",
            "message": "Everything is fine",
            "date": "2024-11-25T12:00:00",
        }
        self.assertEqual(status.to_dict(), expected_dict)



import unittest

from checkout.entities.account import Account
from checkout.entities.credit import Credit
from checkout.entities.instrument import Instrument
from checkout.entities.token import Token


class InstrumentTest(unittest.TestCase):

    def test_instrument_initialization(self):
        """
        Test the initialization of the Instrument class.
        """
        account = Account(bankCode="001", bankName="Test Bank", accountType="Savings", accountNumber="1234567890")
        token = Token(token="test-token", expiry="2025-12-31")
        credit = Credit(code="CRED001", type="Credit", groupCode="GRP001", installment=12)

        instrument = Instrument(bank=account, token=token, credit=credit, pin="1234", password="secure_password")

        self.assertEqual(instrument.bank.bankCode, "001")
        self.assertEqual(instrument.token.token, "test-token")
        self.assertEqual(instrument.credit.code, "CRED001")
        self.assertEqual(instrument.pin, "1234")
        self.assertEqual(instrument.password, "secure_password")

    def test_instrument_to_dict(self):
        """
        Test the `to_dict` method.
        """
        account = Account(bankCode="001", bankName="Test Bank", accountType="Savings", accountNumber="1234567890")
        token = Token(token="test-token", expiry="2025-12-31")
        credit = Credit(code="CRED001", type="Credit", groupCode="GRP001", installment=12)

        instrument = Instrument(bank=account, token=token, credit=credit, pin="1234", password="secure_password")

        expected_dict = {
            "bank": account.to_dict(),
            "token": token.to_dict(),
            "credit": credit.to_dict(),
            "pin": "1234",
            "password": "secure_password",
        }

        self.assertEqual(instrument.to_dict(), expected_dict)

    def test_instrument_to_dict_without_optional_fields(self):
        """
        Test the `to_dict` method when optional fields are not set.
        """
        instrument = Instrument(pin="1234", password="secure_password")

        expected_dict = {
            "bank": None,
            "token": None,
            "credit": None,
            "pin": "1234",
            "password": "secure_password",
        }

        self.assertEqual(instrument.to_dict(), expected_dict)

    def test_instrument_initialization_with_defaults(self):
        """
        Test initialization of the Instrument class with default values.
        """
        instrument = Instrument()

        self.assertIsNone(instrument.bank)
        self.assertIsNone(instrument.token)
        self.assertIsNone(instrument.credit)
        self.assertEqual(instrument.pin, "")
        self.assertEqual(instrument.password, "")



import unittest

from checkout.entities.instrument import Instrument
from checkout.entities.token import Token
from checkout.messages.requests.collect import CollectRequest


class CollectTest(unittest.TestCase):

    def setUp(self):
        self.token_data = {
            "token": "test_token",
            "subtoken": "test_subtoken",
            "franchise": "visa",
            "franchiseName": "Visa",
            "issuerName": "Test Issuer",
            "lastDigits": "1234",
            "validUntil": "2025-12-31",
            "cvv": "123",
            "installments": 12,
        }

        self.token = Token(**self.token_data)
        self.instrument = Instrument(token=self.token, pin="1234", password="secret")
        self.request_data = {
            "instrument": self.instrument,
            "returnUrl": "https://example.com/return",
            "ipAddress": "192.168.1.1",
            "userAgent": "Mozilla/5.0",
        }

    def test_initialization_with_token(self):
        """
        Test CollectRequest initialization with a Token in the Instrument.
        """
        collect_request = CollectRequest(**self.request_data)

        self.assertIsNotNone(collect_request.instrument)
        self.assertIsInstance(collect_request.instrument, Instrument)
        self.assertIsInstance(collect_request.instrument.token, Token)
        self.assertEqual(collect_request.instrument.token.token, self.token_data["token"])
        self.assertEqual(collect_request.instrument.token.franchise, self.token_data["franchise"])
        self.assertEqual(collect_request.instrument.pin, "1234")
        self.assertEqual(collect_request.instrument.password, "secret")
        self.assertEqual(collect_request.return_url, self.request_data["returnUrl"])
        self.assertEqual(collect_request.ip_address, self.request_data["ipAddress"])
        self.assertEqual(collect_request.user_agent, self.request_data["userAgent"])

    def test_to_dict_with_token(self):
        """
        Test CollectRequest to_dict method with a Token in the Instrument.
        """
        collect_request = CollectRequest(**self.request_data)
        result = collect_request.to_dict()

        self.assertIn("instrument", result)
        self.assertEqual(result["instrument"]["token"]["token"], self.token_data["token"])
        self.assertEqual(result["instrument"]["token"]["franchise"], self.token_data["franchise"])
        self.assertEqual(result["instrument"]["pin"], "1234")
        self.assertEqual(result["instrument"]["password"], "secret")

    def test_to_dict_without_token(self):
        """
        Test CollectRequest to_dict method with no Token in the Instrument.
        """
        request_data_no_token = self.request_data.copy()
        request_data_no_token["instrument"] = Instrument(pin="5678", password="no_token")

        collect_request = CollectRequest(**request_data_no_token)
        result = collect_request.to_dict()

        self.assertIn("instrument", result)
        self.assertIsNone(result["instrument"]["token"])
        self.assertEqual(result["instrument"]["pin"], "5678")
        self.assertEqual(result["instrument"]["password"], "no_token")
        self.assertEqual("es_CO", result["locale"])
        self.assertEqual("192.168.1.1", result["ipAddress"])
        self.assertEqual("Mozilla/5.0", result["userAgent"])
        self.assertEqual("https://example.com/return", result["returnUrl"])

        self.assertEqual("false", result["captureAddress"])
        self.assertEqual("false", result["skipResult"])
        self.assertEqual("false", result["noBuyerFill"])



import unittest

from checkout.entities.discount import Discount


class TestDiscount(unittest.TestCase):

    def test_discount_initialization(self):
        discount = Discount(code="DISCOUNT2023", type="Percentage", amount=50.0, base=200.0, percent=25.0)

        self.assertEqual(discount.code, "DISCOUNT2023")
        self.assertEqual(discount.type, "Percentage")
        self.assertEqual(discount.amount, 50.0)
        self.assertEqual(discount.base, 200.0)
        self.assertEqual(discount.percent, 25.0)

    def test_discount_to_dict(self):
        expected_dict = {"code": "DISCOUNT2023", "type": "Percentage", "amount": 50.0, "base": 200.0, "percent": 25.0}

        discount = Discount.model_validate(expected_dict)

        self.assertEqual(discount.to_dict(), expected_dict)

    def test_discount_missing_field(self):
        """
        Test that an error is raised when required fields are missing.
        """
        with self.assertRaises(ValueError):
            Discount(type="Percentage", amount=50.0, base=200.0)

    def test_discount_field_types(self):
        """
        Test that an error is raised when field types are incorrect.
        For example, if 'amount' is a string instead of a float.
        """
        with self.assertRaises(ValueError):
            Discount(code="DISCOUNT2023", type="Percentage", amount="fifty", base=200.0, percent=25.0)



from functools import wraps
from typing import Any, Callable


def convert_booleans_to_strings(func: Callable) -> Callable:
    """
    Decorador para transformar valores booleanos en cadenas "true" o "false".
    """

    @wraps(func)
    def wrapper(*args: tuple, **kwargs: dict[str, Any]) -> Any:
        data = func(*args, **kwargs)
        if isinstance(data, dict):
            return transform_bools(data)
        return data

    return wrapper


def transform_bools(data: Any) -> Any:
    """
    Transforma recursivamente los valores booleanos en "true" o "false".
    """
    if isinstance(data, dict):
        return {k: transform_bools(v) for k, v in data.items()}
    elif isinstance(data, list):
        return [transform_bools(item) for item in data]
    elif isinstance(data, bool):
        return "true" if data else "false"
    return data



import unittest

from checkout.entities.payment_modifier import PaymentModifier


class PaymentModifierTest(unittest.TestCase):

    def test_initialization(self):
        """
        Test initialization of PaymentModifier with default values.
        """
        modifier = PaymentModifier()
        assert modifier.type is None
        assert modifier.code is None
        assert modifier.additional == {}

    def test_initialization_with_values(self):
        """
        Test initialization of PaymentModifier with provided values.
        """
        modifier = PaymentModifier(type="CUSTOM_TYPE", code="MOD123", additional={"key": "value"})
        assert modifier.type == "CUSTOM_TYPE"
        assert modifier.code == "MOD123"
        assert modifier.additional == {"key": "value"}

    def test_set_type(self):
        """
        Test setting the type of PaymentModifier.
        """
        modifier = PaymentModifier()
        modifier.set_type("FEDERAL_GOVERNMENT")
        assert modifier.type == "FEDERAL_GOVERNMENT"

    def test_set_code(self):
        """
        Test setting the code of PaymentModifier.
        """
        modifier = PaymentModifier()
        modifier.set_code("CODE123")
        assert modifier.code == "CODE123"

    def test_get_additional(self):
        """
        Test retrieving additional data by key.
        """
        modifier = PaymentModifier(additional={"key1": "value1", "key2": "value2"})
        assert modifier.get_additional("key1") == "value1"
        assert modifier.get_additional("key3", "default") == "default"

    def test_get_additional_entire_dict(self):
        """
        Test retrieving the entire additional data dictionary.
        """
        additional_data = {"key1": "value1", "key2": "value2"}
        modifier = PaymentModifier(additional=additional_data)
        assert modifier.get_additional() == additional_data

    def test_set_additional(self):
        """
        Test setting additional data.
        """
        modifier = PaymentModifier()
        modifier.set_additional({"key1": "value1", "key2": "value2"})
        assert modifier.additional == {"key1": "value1", "key2": "value2"}

    def test_merge_additional(self):
        """
        Test merging additional data with new data.
        """
        modifier = PaymentModifier(additional={"key1": "value1"})
        modifier.merge_additional({"key2": "value2"})
        assert modifier.additional == {"key1": "value1", "key2": "value2"}

    def test_to_dict(self):
        """
        Test converting PaymentModifier to a dictionary.
        """
        modifier = PaymentModifier(type="CUSTOM_TYPE", code="MOD123", additional={"key": "value"})
        expected_dict = {
            "type": "CUSTOM_TYPE",
            "code": "MOD123",
            "additional": {"key": "value"},
        }
        assert modifier.to_dict() == expected_dict



import unittest

from checkout.entities.amount_detail import AmountDetail


class AmountDetailTest(unittest.TestCase):

    def test_amount_detail_initialization(self):
        amount_detil = AmountDetail(kind="test_kind", amount=10.0)

        assert amount_detil.kind == "test_kind"
        assert amount_detil.amount == 10.0

    def test_amount_detail_to_dict(self):
        expected_dict = {"kind": "test_kind", "amount": 10.0}
        amount_detil = AmountDetail.model_validate(expected_dict)

        assert amount_detil.to_dict() == expected_dict



import unittest

from checkout.entities.status import Status
from checkout.entities.transaction import Transaction
from checkout.enums.status_enum import StatusEnum
from checkout.messages.responses.reverse import ReverseResponse


class ReverseResponseTest(unittest.TestCase):

    def test_initialization_with_all_fields(self):
        """
        Test the initialization of ReverseResponse with all fields provided.
        """
        status = Status(status=StatusEnum.OK, reason="Reversal successful")
        transaction = Transaction(
            reference="REF001",
            internal_reference="INT001",
            payment_method="CreditCard",
            payment_method_name="Visa",
            issuer_name="Bank A",
            authorization="AUTH001",
            receipt="REC001",
            franchise="VISA",
            refunded=True,
        )

        reverse_response = ReverseResponse(
            status=status,
            payment=transaction,
        )

        self.assertIsInstance(reverse_response.status, Status)
        self.assertIsInstance(reverse_response.payment, Transaction)
        self.assertEqual(reverse_response.status.reason, "Reversal successful")
        self.assertEqual(reverse_response.payment.reference, "REF001")

    def test_initialization_with_no_fields(self):
        """
        Test the initialization of ReverseResponse with no fields provided.
        """
        reverse_response = ReverseResponse()

        self.assertIsNone(reverse_response.status)
        self.assertIsNone(reverse_response.payment)

    def test_to_dict_with_all_fields(self):
        """
        Test the to_dict method when all fields are provided.
        """
        status = Status(status=StatusEnum.OK, reason="Reversal successful")
        transaction = Transaction(
            reference="REF001",
            internal_reference="INT001",
            payment_method="CreditCard",
            payment_method_name="Visa",
            issuer_name="Bank A",
            authorization="AUTH001",
            receipt="REC001",
            franchise="VISA",
            refunded=True,
        )

        reverse_response = ReverseResponse(
            status=status,
            payment=transaction,
        )

        expected_dict = {
            "status": status.to_dict(),
            "payment": transaction.to_dict(),
        }

        self.assertEqual(reverse_response.to_dict(), expected_dict)

    def test_to_dict_with_no_fields(self):
        """
        Test the to_dict method when no fields are provided.
        """
        reverse_response = ReverseResponse()

        expected_dict = {
            "status": None,
            "payment": None,
        }

        self.assertEqual(reverse_response.to_dict(), expected_dict)

    def test_to_dict_with_partial_fields(self):
        """
        Test the to_dict method when only one field is provided.
        """
        status = Status(status=StatusEnum.ERROR, reason="Reversal failed")

        reverse_response = ReverseResponse(
            status=status,
        )

        expected_dict = {
            "status": status.to_dict(),
            "payment": None,
        }

        self.assertEqual(reverse_response.to_dict(), expected_dict)



from typing import Dict, Optional, Union

from pydantic import BaseModel, ConfigDict, Field

from checkout.entities.amount_base import AmountBase


class AmountConversion(BaseModel):

    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    fromAmount: Optional[AmountBase] = Field(default=None, description="Base amount to convert from", alias="from")
    toAmount: Optional[AmountBase] = Field(default=None, description="Base amount to convert to", alias="to")
    factor: float = Field(default=1.0, description="Conversion factor")

    def set_amount_base(self, base: Union[Dict, AmountBase]) -> None:
        """
        Quickly set all values to the same base.

        :param base: Either a dictionary or an instance of AmountBase.
        """
        amount_base = AmountBase(**base) if isinstance(base, dict) else base
        self.toAmount = amount_base
        self.fromAmount = amount_base
        self.factor = 1.0

    def to_dict(self) -> dict:
        """
        Convert the AmountConversion object to a dictionary.
        """
        return {
            "fromAmount": self.fromAmount.to_dict() if self.fromAmount else None,
            "toAmount": self.toAmount.to_dict() if self.toAmount else None,
            "factor": self.factor,
        }
